//
// Created by zengwenna on 2018/12/27.
//

#include <jni.h>
#include <string>
#include <android/log.h>

extern "C" JNIEXPORT jstring

#define LOG_TAG "NativeUtils-Jni"
#define LOGD(...) __android_log_print(ANDROID_LOG_DEBUG,LOG_TAG, __VA_ARGS__)
#define LOGE(...) __android_log_print(ANDROID_LOG_ERROR,LOG_TAG, __VA_ARGS__)

#define REGISTER_CLASS "com/orbbec/obviewer2/util/NativeUtils"


JNICALL
Java_com_orbbec_obviewer2_util_copyByteBuffer(
        JNIEnv *env,
jobject /* this */) {
std::string hello = "Hello from C++";
return env->NewStringUTF(hello.c_str());
}

int *m_histogram;
enum {
    HISTSIZE = 0xFFFF,
};

int ConventFromDepthToRGB(short *src, int *dst, int w, int h, int strideInBytes) {

    // Calculate the accumulative histogram (the yellow display...)
    if (m_histogram == NULL) {
        m_histogram = new int[HISTSIZE];
    }
    memset(m_histogram, 0, HISTSIZE * sizeof(int));

    int nNumberOfPoints = 0;
    unsigned int value;
    int Size = w * h;
    for (int i = 0; i < Size; ++i) {
        value = src[i];
        if (value != 0) {
            m_histogram[value]++;
            nNumberOfPoints++;
        }
    }

    int nIndex;
    for (nIndex = 1; nIndex < HISTSIZE; nIndex++) {
        m_histogram[nIndex] += m_histogram[nIndex - 1];
    }

    if (nNumberOfPoints != 0) {
        for (nIndex = 1; nIndex < HISTSIZE; nIndex++) {
            m_histogram[nIndex] = (unsigned int) (256 * (1.0f - ((float) m_histogram[nIndex] / nNumberOfPoints)));
        }
    }

    for (int y = 0; y < h; ++y) {
        uint8_t *rgb = (uint8_t *) (dst + y * w);
        short *pView = src + y * w;
        for (int x = 0; x < w; ++x, rgb += 3, pView++) {
            value = m_histogram[*pView];
            rgb[0] = value;
            rgb[1] = value;
            rgb[2] = 0x00;
        }
    }
    return 0;
}

jint ConvertDepthBytesTORGB(JNIEnv *env, jobject obj, jobject src, jbyteArray dst, jint w, jint h,
                             jint strideInBytes){
    if (src == NULL || dst == NULL) {
        return -1;
    }
    short *srcBuf = (short *) env->GetDirectBufferAddress(src);
    int *dstBuf = (int *) env->GetDirectBufferAddress(dst);
    ConventFromDepthToRGB(srcBuf, dstBuf, w, h, strideInBytes);
    return 0;
}

jint ByteBufferCopy(JNIEnv *env, jobject obj, jobject srcData,  jobject dstData, jint dataLength){

    if (srcData == NULL || dstData == NULL || dataLength <= 0){
        LOGD("ByteBufferCopy(), Illegal parameter");
        return 0;
    }
    unsigned char *src = (unsigned char *) (*env).GetDirectBufferAddress(srcData);
    unsigned char *dst = (unsigned char *) (*env).GetDirectBufferAddress(dstData);
    memcpy(dst, src, dataLength * sizeof(unsigned char));
    return 0;
}

JNINativeMethod jniMethods[] = {
        {"ConvertTORGB", "(Ljava/nio/ByteBuffer;Ljava/nio/ByteBuffer;III)I",     (void *) &ConvertDepthBytesTORGB},
        {"ByteBufferCopy",      "(Ljava/nio/ByteBuffer;Ljava/nio/ByteBuffer;I)I",  (void *)&ByteBufferCopy}
};

jint JNI_OnLoad(JavaVM *vm, void *reserved) {
    JNIEnv *env;
    jclass gCallbackClass = NULL;

    if (vm->GetEnv(reinterpret_cast<void **>(&env), JNI_VERSION_1_6) != JNI_OK) {
        return -1;
    }

    jclass clz = env->FindClass(REGISTER_CLASS);
    gCallbackClass = (jclass) env->NewGlobalRef(clz);
    env->RegisterNatives(clz, jniMethods, sizeof(jniMethods) / sizeof(JNINativeMethod));
    env->DeleteLocalRef(clz);

    LOGD("DepthUtils JNI_OnLoad");
    return JNI_VERSION_1_6;
}