package com.orbbec.obviewer2;

import android.Manifest;
import android.content.pm.PackageManager;
import android.graphics.ImageFormat;
import android.graphics.PixelFormat;
import android.opengl.GLSurfaceView;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.support.annotation.NonNull;
import android.support.v4.app.ActivityCompat;
import android.support.v7.app.AppCompatActivity;
import android.util.Log;
import android.view.KeyEvent;
import android.view.View;
import android.view.ViewTreeObserver;
import android.view.Window;
import android.widget.CheckBox;
import android.widget.RadioButton;
import android.widget.RadioGroup;
import android.widget.TextView;
import android.widget.Toast;
import android.widget.ToggleButton;

import com.orbbec.astra.UsbDeviceInfo;
import com.orbbec.astra.Vector2D;
import com.orbbec.obviewer2.callback.ViewCallBack;
import com.orbbec.obviewer2.listener.CheckBoxCheckChangeListenter;
import com.orbbec.obviewer2.model.OBSourceInterface;
import com.orbbec.obviewer2.present.ObViewWallPresentImp;
import com.orbbec.obviewer2.present.OpenInterface;
import com.orbbec.obviewer2.util.FPSMeter;
import com.orbbec.obviewer2.util.NativeUtils;
import com.orbbec.obviewer2.util.SelfDialog;
import com.orbbec.obviewer2.util.Utils;
import com.orbbec.obviewer2.view.GlBodyRenderer;
import com.orbbec.obviewer2.view.GlBodyView;
import com.orbbec.obviewer2.view.OpenGLMaskColorView;
import com.orbbec.obviewer2.view.OpenGlColorView;
import com.orbbec.obviewer2.view.OpenGlColorizedBodyView;
import com.orbbec.obviewer2.view.OpenGlDepthView;

import java.lang.ref.WeakReference;
import java.nio.ByteBuffer;
import java.nio.IntBuffer;
import java.nio.ShortBuffer;

import static android.os.Build.VERSION_CODES.M;
import static android.view.View.SYSTEM_UI_FLAG_IMMERSIVE;

public class MainActivity extends AppCompatActivity implements ViewCallBack {

    public static final String TAG = "lgp";
    private OpenGlColorView mColorView;
    private OpenGlDepthView mDepthView;
    private OpenGLMaskColorView mMaskedColorView;
    private OpenGlColorizedBodyView mColorizedBodyView;
    private GlBodyView mGLBodyView;
    private GlBodyRenderer mGLBodyRenderer;
    private TextView mTxtCpuRate;
    private TextView mTxtFRCurrent;
    private TextView mTxtAppMemory;

    private TextView body_fps;
    private TextView text_fps;


//    private Button mSaveButton;

    private int mColorWidth = 640;
    private int mColorHeight = 480;

    private int mDepthHeight = 480;
    private int mDepthWidth = 640;

    short[] mDepthPixel;
    int[] mDepthHist;
    private OpenInterface openInterface;
    private MainHandler mHandler;


    static class MainHandler extends Handler {
        WeakReference<MainActivity> mActivity;

        public MainHandler(MainActivity activity) {
            mActivity = new WeakReference<MainActivity>(activity);
        }

        @Override
        public void handleMessage(Message msg) {
            //每隔1S保存一次
            switch (msg.what) {
                case 1:
                    Utils.getInstance().calculateRunTime();
                    MainActivity mainActivity = mActivity.get();
                    mainActivity.setCalculateRunTimeText(Utils.getInstance().getBuild());
                    mainActivity.calculateCPU();
                    mainActivity.calculateRAM();
                    mainActivity.calulateFPS();
                    break;
                default:
                    break;
            }
        }
    }

    public void setCalculateRunTimeText(String result) {

    }

    public void calulateFPS() {
        Utils.getInstance().calulateFPS(text_fps);
    }

    public void calculateCPU() {
        Utils.getInstance().calculateCPU(
                mTxtCpuRate, mTxtAppMemory);
    }

    public void calculateRAM() {
//todo
        Utils.getInstance().calculateRAM(mTxtAppMemory);
    }

    private Runnable calculatorRunInfo = new Runnable() {
        @Override
        public void run() {
            Message message = new Message();
            message.what = 1;
            mHandler.sendMessage(message);

            mHandler.removeCallbacks(this);
            //todo 这个地方消耗资源过多
            mHandler.postDelayed(this, 100);
        }
    };


    private ToggleButton btn_depth;
    private ToggleButton btn_color;
    private ToggleButton btn_bady;
    private ToggleButton btn_maskcolor;
    private ToggleButton btn_colorizedbody;
    private CheckBox depth_mirror;
    private CheckBox color_mirror;
    private RadioGroup depth_resolution;
    private RadioButton depth_160;
    private RadioButton depth_320;
    private RadioButton depth_640;
    private RadioButton depth_400;

    private RadioGroup color_resolution;
    private RadioButton color_160;
    private RadioButton color_320;
    private RadioButton color_640;
    private RadioButton color_1280;
    private RadioButton color_1920;

    private RadioGroup skeleton_profile;
    private RadioButton skeleton_full;
    private RadioButton skeleton_upper_body;
    private RadioButton skeleton_basic;

    private RadioGroup skeleton_features;
    private RadioButton segment;
    private RadioButton segment_skeleton;
    private RadioButton segment_skeleton_hand;

    private ToggleButton transcribe;
    private static int BUS_CL_PID = 1552;


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        requestWindowFeature(Window.FEATURE_NO_TITLE);
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main2);

        mHandler = new MainHandler(this);
        View decorView = getWindow().getDecorView();
        int uiOptions = View.SYSTEM_UI_FLAG_HIDE_NAVIGATION | View.SYSTEM_UI_FLAG_FULLSCREEN | SYSTEM_UI_FLAG_IMMERSIVE;
        decorView.setSystemUiVisibility(uiOptions);
        initDataView();
        openInterface = new ObViewWallPresentImp(this);
        ((ObViewWallPresentImp) openInterface).setViewCallBack(this);
        ((ObViewWallPresentImp) openInterface).setmGLBodyRenderer(mGLBodyRenderer);
        setCheckBoxChoice();
        //获取权限
        if (Build.VERSION.SDK_INT >= M) {
            if ((ActivityCompat.checkSelfPermission(this, Manifest.permission.CAMERA) | ActivityCompat.checkSelfPermission(this, Manifest.permission.WRITE_EXTERNAL_STORAGE))
                    != PackageManager.PERMISSION_GRANTED) {
                requestPermissions(new String[]{Manifest.permission.WRITE_EXTERNAL_STORAGE, Manifest.permission.CAMERA}, 0);
            } else if (checkPermission()) {
                openInterface.initDevice();
                hideBottomUIMenu();
            }
        } else {
            openInterface.initDevice();
            hideBottomUIMenu();
        }


        Log.e("NativeUtils", "NativeUtils.ByteBufferCopy() = " + NativeUtils.ByteBufferCopy(null, null, 0));


    }

    private void initDataView() {
        mDepthHist = new int[mDepthWidth * mDepthHeight];
        mDepthPixel = new short[mDepthWidth * mDepthHeight];
//        mDepthBuffer = IntBuffer.allocate(mDepthWidth * mDepthHeight);
        //  mTxtCPUTotal = findViewById(R.id.txt_cpu_total);
        mTxtCpuRate = findViewById(R.id.cpu_rate);
        mTxtAppMemory = findViewById(R.id.app_memory);
        // mTxtCPUAverage = findViewById(R.id.txt_cpu_average);
        //  mTxtCPUAverage = findViewById(R.id.cpu_current);
        //  mTxtFRCurrent = findViewById(R.id.txt_frupdate_current);
        mTxtFRCurrent = findViewById(R.id.skeleton_fps);
        text_fps = findViewById(R.id.text_fps);
        body_fps = findViewById(R.id.body_fps);
        initGLView();
        initViewState();


    }

    private void initGLView() {
        mColorView = findViewById(R.id.colorview);
        mDepthView = findViewById(R.id.depthview);
        mMaskedColorView = findViewById(R.id.maskcolorview);
        mColorizedBodyView = findViewById(R.id.colorizedbodyview);

        mGLBodyView = findViewById(R.id.skeletonview);
        mGLBodyView.setEGLContextClientVersion(2);
        //设置背景颜色透明
        mGLBodyView.setEGLConfigChooser(8, 8, 8, 8, 16, 0);
        mGLBodyView.getHolder().setFormat(PixelFormat.TRANSLUCENT);
        mGLBodyView.setZOrderOnTop(true);

        mGLBodyRenderer = new GlBodyRenderer(this);
        mGLBodyView.setRenderer(mGLBodyRenderer);
        mGLBodyView.setRenderMode(GLSurfaceView.RENDERMODE_WHEN_DIRTY);
    }

    private void initViewState() {
        mColorView.setVisibility(View.INVISIBLE);
        mDepthView.setVisibility(View.INVISIBLE);
        mGLBodyView.setVisibility(View.INVISIBLE);
        mMaskedColorView.setVisibility(View.INVISIBLE);
        mColorizedBodyView.setVisibility(View.INVISIBLE);
    }

    private void setCheckBoxChoice() {
        /**
         * 新增按钮id
         */
        btn_depth = findViewById(R.id.btn_depth);

        btn_color = findViewById(R.id.btn_Color);
        btn_bady = findViewById(R.id.btn_Body);
        btn_maskcolor = findViewById(R.id.btn_MaskColor);
        btn_colorizedbody = findViewById(R.id.btn_ColorizedBody);
        depth_mirror = findViewById(R.id.depth_mirrored);
        color_mirror = findViewById(R.id.color_mirror);
        depth_resolution = findViewById(R.id.depth_resolution);
        depth_160 = findViewById(R.id.depth_160);
        depth_320 = findViewById(R.id.depth_320);
        depth_640 = findViewById(R.id.depth_640);
        color_resolution = findViewById(R.id.color_resolution);
        color_160 = findViewById(R.id.color_160);
        color_320 = findViewById(R.id.color_320);
        color_640 = findViewById(R.id.color_640);
        color_1280 = findViewById(R.id.color_1280);
        color_1920 = findViewById(R.id.color_1920);

        skeleton_profile = findViewById(R.id.skeleton_profile);
        skeleton_full = findViewById(R.id.skeleton_full);
        skeleton_upper_body = findViewById(R.id.skeleton_upper_body);
        skeleton_basic = findViewById(R.id.skeleton_basic);

        skeleton_features = findViewById(R.id.skeleton_features);
        segment = findViewById(R.id.segment);
        segment_skeleton = findViewById(R.id.segment_skeleton);
        segment_skeleton_hand = findViewById(R.id.segment_skeleton_hand);
        transcribe = findViewById(R.id.transcribe);

        CheckBoxCheckChangeListenter checkBoxCheckChangeListenter = new CheckBoxCheckChangeListenter(openInterface,
                mColorView, mDepthView,
                mMaskedColorView, mColorizedBodyView,
                mGLBodyView, this);

        //  mToggleDepthRecord.setOnCheckedChangeListener(checkBoxCheckChangeListenter);

        btn_depth.setOnCheckedChangeListener(checkBoxCheckChangeListenter);

        btn_color.setOnCheckedChangeListener(checkBoxCheckChangeListenter);

        btn_bady.setOnCheckedChangeListener(checkBoxCheckChangeListenter);
        btn_maskcolor.setOnCheckedChangeListener(checkBoxCheckChangeListenter);
        btn_colorizedbody.setOnCheckedChangeListener(checkBoxCheckChangeListenter);
        depth_mirror.setOnCheckedChangeListener(checkBoxCheckChangeListenter);
        color_mirror.setOnCheckedChangeListener(checkBoxCheckChangeListenter);
        depth_resolution.setOnCheckedChangeListener(checkBoxCheckChangeListenter);
        color_resolution.setOnCheckedChangeListener(checkBoxCheckChangeListenter);

        skeleton_profile.setOnCheckedChangeListener(checkBoxCheckChangeListenter);
        skeleton_features.setOnCheckedChangeListener(checkBoxCheckChangeListenter);
        transcribe.setOnCheckedChangeListener(checkBoxCheckChangeListenter);

    }


    @Override
    protected void onResume() {
        super.onResume();
        mHandler.post(calculatorRunInfo);

    }

    @Override
    protected void onPause() {
        super.onPause();

        mHandler.removeCallbacks(calculatorRunInfo);

    }

    @Override
    protected void onStop() {
        openInterface.onRelease();
        super.onStop();
    }

    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event) {
        if (keyCode == KeyEvent.KEYCODE_BACK) {
            openInterface.onRelease();
            finish();
            System.exit(0);
        } else if (keyCode == KeyEvent.KEYCODE_HOME) {
            openInterface.onRelease();
            finish();
            System.exit(0);
        }
        return false;
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        if (grantResults[0] == PackageManager.PERMISSION_GRANTED) {
            openInterface.initDevice();
        } else {
            Toast.makeText(this, "App will exit in 3 seconds", Toast.LENGTH_LONG).show();
            try {
                Thread.sleep(3000);
                finish();
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }
    }

    private boolean checkPermission() {
        return (ActivityCompat.checkSelfPermission(this, Manifest.permission.CAMERA) == PackageManager.PERMISSION_GRANTED &&
                ActivityCompat.checkSelfPermission(this, Manifest.permission.WRITE_EXTERNAL_STORAGE) == PackageManager.PERMISSION_GRANTED);
    }

    @Override
    public void update(final int type, int width, int height, final ByteBuffer buffer) {

        switch (type) {
            case OBSourceInterface.TYPE_COLOR:
                mColorView.update(width, height, buffer);
                break;
            case OBSourceInterface.TYPE_BODY:

                mGLBodyView.requestRender();
                break;
            case OBSourceInterface.TYPE_COLOR_ZEDBODY:

                mColorizedBodyView.update(width, height, buffer);
                break;
            case OBSourceInterface.TYPE_MASKED_COLOR:
                mMaskedColorView.update(width, height, buffer);
                break;
            default:
                break;
        }
    }


    @Override
    public void updateBodyData(final int index, final Vector2D pos, final int depth) {
        runOnUiThread(new Runnable() {
            @Override
            public void run() {
                switch (index) {
                    case -1:

                        break;
                    case 0:

                        break;

                    case 1:

                        break;

                    case 2:

                        break;

                    case 3:

                        break;

                    case 4:

                        break;
                }
            }
        });

    }


    @Override
    public void updateDepth(int width, int height, ShortBuffer depthData) {
//        runOnUiThread(new Runnable() {
//            @Override
//            public void run() {
//            }
//        });

        if (mDepthPixel.length != width * height) {
            mDepthPixel = new short[width * height];
        }
        depthData.get(mDepthPixel);
        int index = 0;

        for (int y = 0; y < height; y++) {
            for (int x = 0; x < width; x++) {
                int depth = mDepthPixel[width * y + x];

                mDepthHist[index] = 0xff000000 |
                        (depth * 255 / 8000) << 16 |
                        (depth * 255 / 8000) << 8 |
                        (depth * 255 / 8000);
                index++;
            }
        }

//        mDepthBuffer.clear();
//        mDepthBuffer.put(mDepthHist);
        IntBuffer mDepthBuffer = IntBuffer.wrap(mDepthHist);
        mDepthView.fillTexture(width, height, mDepthBuffer);

//        mDepthView.update(mDepthWidth, mDepthHeight, depthData);
    }

    @Override
    public void calculateUpdateData(FPSMeter colorMeter) {
//        Utils.getInstance().calculateUpdateData(colorMeter, mTxtUpdateDataCurrent,
//                mTxtUpdateDataAverage, mTxtUpdateDataMinimum,
//                mTxtUpdateDataHighest, mTxtBKUpdateDataCurrent,
//                mTxtBKUpdateDataAverage, mTxtBKUpdateDataMinimum,
//                mTxtBKUpdateDataHighest);
    }

    @Override
    public void calculateShowTime() {
        Utils.getInstance().calculateShowTime(mColorView, mDepthView,
                mMaskedColorView, mColorizedBodyView,
                mGLBodyRenderer, mTxtFRCurrent, text_fps);
    }

    @Override
    public void isCameraOpen() {
        Log.d(TAG, "isDepthCameraOpen: ");
        btn_depth.setChecked(true);
        Log.d(TAG, "isCameraOpen: " + colorPid);
        if (colorPid == BUS_CL_PID) {
            btn_color.setEnabled(false);
            btn_color.setBackgroundColor(getResources().getColor(R.color.btn_no_enabled));
            btn_maskcolor.setEnabled(false);
            btn_maskcolor.setBackgroundColor(getResources().getColor(R.color.btn_no_enabled));
            btn_colorizedbody.setEnabled(false);
            btn_colorizedbody.setBackgroundColor(getResources().getColor(R.color.btn_no_enabled));
            //RGB相关要屏蔽掉，设置不可点击的样式
            color_mirror.setEnabled(false);
            color_320.setEnabled(false);
            color_640.setEnabled(false);
            color_1280.setEnabled(false);
            color_1920.setEnabled(false);
        } else {
            btn_color.setChecked(true);
        }

    }

    @Override
    public void CameraOpenFailed() {
        Log.d(TAG, "CameraOpenFailed: ");
        Utils.getInstance().CameraOpenFailedDialog(this);
    }

    @Override
    public void onNoDevice() {
        Utils.getInstance().CameraOpenFailedDialog(this);
    }

    private static int colorPid;

    @Override
    public void isExistsColorCamera(int pid) {
        colorPid = pid;
    }

    /**
     * /隐藏虚拟按键，并且全屏
     */
    protected void hideBottomMenu() {
        //隐藏虚拟按键，并且全屏
        if (Build.VERSION.SDK_INT > 11 && Build.VERSION.SDK_INT < 19) { // lower api
            View v = this.getWindow().getDecorView();
            v.setSystemUiVisibility(View.GONE);
        } else if (Build.VERSION.SDK_INT >= 19) {
            //for new api versions.
            View decorView = getWindow().getDecorView();
            int uiOptions = View.SYSTEM_UI_FLAG_HIDE_NAVIGATION
                    | View.SYSTEM_UI_FLAG_IMMERSIVE_STICKY
                    | View.SYSTEM_UI_FLAG_FULLSCREEN;
            decorView.setSystemUiVisibility(uiOptions);
        }
    }


    protected void hideBottomUIMenu() {
        int flags;
        int curApiVersion = android.os.Build.VERSION.SDK_INT;
        // This work only for android 4.4+
        if (curApiVersion >= Build.VERSION_CODES.KITKAT) {

            // This work only for android 4.4+
            // hide navigation bar permanently in android activity
            // touch the screen, the navigation bar will not show

            flags = View.SYSTEM_UI_FLAG_FULLSCREEN
                    | View.SYSTEM_UI_FLAG_HIDE_NAVIGATION
                    | View.SYSTEM_UI_FLAG_IMMERSIVE
                    | View.SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN
                    | View.SYSTEM_UI_FLAG_LAYOUT_HIDE_NAVIGATION;
        } else {
            // touch the screen, the navigation bar will show
            flags = View.SYSTEM_UI_FLAG_HIDE_NAVIGATION |
                    View.SYSTEM_UI_FLAG_LAYOUT_HIDE_NAVIGATION;
        }

        // must be executed in main thread :)
        getWindow().getDecorView().setSystemUiVisibility(flags);
    }


}


