package com.orbbec.obviewer2.listener;

import android.content.Context;
import android.os.Environment;
import android.text.format.DateFormat;
import android.util.Log;
import android.view.View;
import android.widget.CompoundButton;
import android.widget.RadioGroup;

import com.orbbec.astra.BodyTrackingFeatureFlags;
import com.orbbec.astra.SkeletonProfile;

import android.widget.Toast;

import com.orbbec.astra.DepthStream;
import com.orbbec.obviewer2.R;
import com.orbbec.obviewer2.model.OBSourceInterface;
import com.orbbec.obviewer2.present.OpenInterface;
import com.orbbec.obviewer2.util.IToast;
import com.orbbec.obviewer2.view.GlBodyRenderer;

import static com.orbbec.obviewer2.MainActivity.TAG;

import java.util.Date;

/**
 * @author tanzhuohui
 * @date 2018/12/26
 */
public class CheckBoxCheckChangeListenter implements CompoundButton.OnCheckedChangeListener, RadioGroup.OnCheckedChangeListener {

    private Context mContext;
    private OpenInterface openInterface;
    private View mColorView;
    private View mDepthView;
    private View mMaskedColorView;
    private View mColorizedBodyView;
    private View mGLBodyView;
    private Context context;
    private DepthStream stream;
    private GlBodyRenderer glBodyRenderer;

    public CheckBoxCheckChangeListenter(OpenInterface openInterface, View mColorView, View mDepthView,
                                        View mMaskedColorView, View mColorizedBodyView,
                                        View mGLBodyView, Context context) {
        this.mContext = context;
        this.openInterface = openInterface;
        this.mColorView = mColorView;
        this.mDepthView = mDepthView;
        this.mMaskedColorView = mMaskedColorView;
        this.mColorizedBodyView = mColorizedBodyView;
        this.mGLBodyView = mGLBodyView;
        this.context = context;
        glBodyRenderer = new GlBodyRenderer(context);
    }


    @Override
    public void onCheckedChanged(CompoundButton compoundButton, boolean isChecked) {


        switch (compoundButton.getId()) {

            case R.id.btn_depth: {

                boolean checked = compoundButton.isChecked();
                operatingStream(mDepthView, checked, OBSourceInterface.TYPE_DEPTH);
                break;
            }
            case R.id.btn_Color: {
                boolean checked = compoundButton.isChecked();
                operatingStream(mColorView, checked, OBSourceInterface.TYPE_COLOR);
                break;
            }
            case R.id.btn_Body: {
                //todo 开启 彩色抠图或者深度背景抠图时 骨骼不能关闭
                boolean checked = compoundButton.isChecked();
                operatingStream(mGLBodyView, checked, OBSourceInterface.TYPE_BODY);
                break;
            }
            case R.id.btn_MaskColor: {
                boolean checked = compoundButton.isChecked();
                operatingStream(mMaskedColorView, checked, OBSourceInterface.TYPE_MASKED_COLOR);
                break;
            }
            case R.id.btn_ColorizedBody: {
                boolean checked = compoundButton.isChecked();
                operatingStream(mColorizedBodyView, checked, OBSourceInterface.TYPE_COLOR_ZEDBODY);
                break;
            }

            case R.id.depth_mirrored: {
                boolean checked = compoundButton.isChecked();
                openInterface.setMirror(OBSourceInterface.TYPE_DEPTH, checked);
                break;
            }
            case R.id.color_mirror: {
                boolean checked = compoundButton.isChecked();
                openInterface.setMirror(OBSourceInterface.TYPE_COLOR, checked);
                break;
            }


            case R.id.bt_color:
                operatingStream(mColorView, isChecked, OBSourceInterface.TYPE_COLOR);
                break;
            case R.id.bt_depth:
                operatingStream(mDepthView, isChecked, OBSourceInterface.TYPE_DEPTH);
                break;
            case R.id.bt_maskedcolor:
                operatingStream(mMaskedColorView, isChecked, OBSourceInterface.TYPE_MASKED_COLOR);
                break;
            case R.id.bt_colorizedbody:
                operatingStream(mColorizedBodyView, isChecked, OBSourceInterface.TYPE_COLOR_ZEDBODY);
                break;
            case R.id.bt_skeleton:
                operatingStream(mGLBodyView, isChecked, OBSourceInterface.TYPE_BODY);
                break;
            case R.id.bt_iscolormirrored: {
                boolean checked = compoundButton.isChecked();
                openInterface.setMirror(OBSourceInterface.TYPE_COLOR, checked);
                break;
            }

            case R.id.transcribe: {
                boolean checked = compoundButton.isChecked();
                Log.d(TAG, "onClick: transcribe" + checked);
                if (checked) {
                    Log.d(TAG, "onClick: transcribe");
                    stream = (DepthStream) openInterface.getStream(OBSourceInterface.TYPE_DEPTH);
                    String filename = Environment.getExternalStorageDirectory().getPath() + '/' + DateFormat.format("HH-mm-ss.oni", new Date());
                    //todo 视频录制.
                    Log.d(TAG, "onCheckedChanged: " + filename);
                    stream.startRecord(filename);
                } else {
                    stream.stopRecord();
                    Log.d(TAG, "onCheckedChanged: 停");
                    IToast.show(context);
                }
            }
            break;

            case R.id.tb_depthres: {
                boolean checked = compoundButton.isChecked();
                if (checked) {
                    openInterface.setResolution(OBSourceInterface.TYPE_DEPTH, 640, 400);
                } else {
                    openInterface.setResolution(OBSourceInterface.TYPE_DEPTH, 640, 480);
                }
                break;
            }
            case R.id.tb_depthrecord: {
                boolean checked = compoundButton.isChecked();
                DepthStream stream = (DepthStream) openInterface.getStream(OBSourceInterface.TYPE_DEPTH);
                String filename = Environment.getExternalStorageDirectory().getPath() + '/' + DateFormat.format("HH-mm-ss.oni", new Date());
                if (checked) {
                    Toast.makeText(mContext, "save to " + filename, Toast.LENGTH_LONG).show();
                    stream.startRecord(filename);
                } else {
                    stream.stopRecord();
                }
                break;
            }
            case R.id.bt_isshow:
                operatingStream(null, isChecked, OBSourceInterface.TYPE_SHOW_STREAM);
                break;
            default:
                break;
        }
    }

    //统一控制流
    private void operatingStream(View view, boolean isCheck, int type) {
        if (isCheck) {
            Log.i("CheckBox", "mCheckColorBox is selected");
            view.setVisibility(View.VISIBLE);
            openInterface.startStream(type);
            openInterface.addType(type);
        } else {
            Log.i("CheckBox", "mCheckColorBox is cancelled");
            view.setVisibility(View.INVISIBLE);
            openInterface.stopStream(type);
            openInterface.cutType(type);
        }
    }

    /**
     * @param view
     * @param type 在开启的状态 切换分辨率
     */
    private void operatingStreamResolution(View view, int type) {


        int isshow = view.getVisibility();
        Log.d(TAG, "operatingStreamResolution: " + isshow);
        if (isshow == 0) {
            openInterface.startStream(type);
        }
    }


    /**
     * radio group 事件处理
     *
     * @param radioGroup
     * @param i
     */
    @Override
    public void onCheckedChanged(RadioGroup radioGroup, int i) {

        switch (radioGroup.getCheckedRadioButtonId()) {
//depth分辨率设置
            case R.id.depth_160: {
                openInterface.setResolution(OBSourceInterface.TYPE_DEPTH, 160, 120);
                operatingStreamResolution(mDepthView, OBSourceInterface.TYPE_DEPTH);
                break;
            }
            case R.id.depth_320: {
                openInterface.setResolution(OBSourceInterface.TYPE_DEPTH, 320, 240);
                operatingStreamResolution(mDepthView, OBSourceInterface.TYPE_DEPTH);
                break;
            }
            case R.id.depth_640: {
                openInterface.setResolution(OBSourceInterface.TYPE_DEPTH, 640, 480);
                operatingStreamResolution(mDepthView, OBSourceInterface.TYPE_DEPTH);
                break;
            }

            case R.id.depth_400: {
                openInterface.setResolution(OBSourceInterface.TYPE_DEPTH, 640, 400);
                operatingStreamResolution(mDepthView, OBSourceInterface.TYPE_DEPTH);
                break;
            }


//RGB分辨率设置
            case R.id.color_160: {
                Log.d(TAG, "onCheckedChanged: " + R.id.color_160 + " : " + radioGroup.getCheckedRadioButtonId());
                openInterface.setResolution(OBSourceInterface.TYPE_COLOR, 160, 120);
                operatingStreamResolution(mColorView, OBSourceInterface.TYPE_COLOR);
                break;
            }
            case R.id.color_320: {
                openInterface.setResolution(OBSourceInterface.TYPE_COLOR, 320, 240);
                operatingStreamResolution(mColorView, OBSourceInterface.TYPE_COLOR);
                break;
            }
            case R.id.color_640: {
                openInterface.setResolution(OBSourceInterface.TYPE_COLOR, 640, 480);
                operatingStreamResolution(mColorView, OBSourceInterface.TYPE_COLOR);
                break;
            }
            case R.id.color_1280: {
                openInterface.setResolution(OBSourceInterface.TYPE_COLOR, 1280, 720);
                operatingStreamResolution(mColorView, OBSourceInterface.TYPE_COLOR);
                break;
            }
            case R.id.color_1920: {
                openInterface.setResolution(OBSourceInterface.TYPE_COLOR, 1920, 1080);
                operatingStreamResolution(mColorView, OBSourceInterface.TYPE_COLOR);

                break;
            }
            //骨架模式设置
            case R.id.skeleton_full: {
                openInterface.setSkeletonProfile(SkeletonProfile.FULL);
                glBodyRenderer.modeBody(0);

                break;
            }
            case R.id.skeleton_upper_body: {
                openInterface.setSkeletonProfile(SkeletonProfile.UPPER_BODY);
                glBodyRenderer.modeBody(1);
                break;
            }
            case R.id.skeleton_basic: {
                openInterface.setSkeletonProfile(SkeletonProfile.BASIC);
                glBodyRenderer.modeBody(2);
                break;
            }
//骨架特征设置
            case R.id.segment:
                openInterface.setDefaultBodyFeatures(BodyTrackingFeatureFlags.SEGMENTATION);
                break;
            case R.id.segment_skeleton:
                openInterface.setDefaultBodyFeatures(BodyTrackingFeatureFlags.JOINTS);
                break;
            case R.id.segment_skeleton_hand:
                openInterface.setDefaultBodyFeatures(BodyTrackingFeatureFlags.HAND_POSES);
                break;

            default:
                break;

        }
    }


}
