package com.orbbec.obviewer2.listener;

import android.app.Activity;
import android.content.Context;
import android.os.Environment;
import android.util.Log;

import com.orbbec.astra.BodyFrame;
import com.orbbec.astra.ColorFrame;
import com.orbbec.astra.ColorizedBodyFrame;
import com.orbbec.astra.DepthFrame;
import com.orbbec.astra.MaskedColorFrame;
import com.orbbec.astra.ReaderFrame;
import com.orbbec.astra.StreamReader;
import com.orbbec.obviewer2.model.OBSourceInterface;
import com.orbbec.obviewer2.present.OpenInterface;
import com.orbbec.obviewer2.util.FPSMeter;
import com.orbbec.obviewer2.util.Utils;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import static com.orbbec.obviewer2.MainActivity.TAG;

/**
 * @author tanzhuohui
 * @date 2018/12/25
 */
public class OBFrameListener implements StreamReader.FrameListener {
    private static final String DEVICEINFO_PATH = Environment.getExternalStorageDirectory().getPath() + "/deviceInfo.txt";
    private Context context;
    private long mLastTimes = 0L;
    private FPSMeter colorMeter;
    private List<Integer> types;
    private boolean isNewFile = true;
    private OBSourceInterface obSourceInterface;
    private File tmpFile = null;
    boolean mHasUser = false;
    private OpenInterface openInterface;

    public OBFrameListener(OpenInterface openInterface, OBSourceInterface obSourceInterface, Context context) {
        this.context = context;
        this.obSourceInterface = obSourceInterface;
        colorMeter = new FPSMeter();
        this.openInterface = openInterface;
    }

    public void setTypes(ArrayList<Integer> types) {
        this.types = types;
    }

    @Override
    public void onFrameReady(StreamReader reader, ReaderFrame frame) {
        colorMeter.mesureFps();
        ((Activity) context).runOnUiThread(new Runnable() {
            @Override
            public void run() {
                long currentTime = System.currentTimeMillis();
                //save per seconds
                if (currentTime - mLastTimes > 100) {
                    //calculate fps and used time
                    openInterface.calculateUpdateData(colorMeter);
                    openInterface.calculateShowTime();
                    //show time in front
                    if (Utils.getInstance().ismSaveFrontInfo()) {
                        String saveCurrentTime = "time:" + Utils.getInstance().currentTime() + "\t";
                        Log.d(TAG, "run: " + Utils.getInstance().getmDeviceInfo());
                        Utils.getInstance().saveStateData(DEVICEINFO_PATH, saveCurrentTime + Utils.getInstance().getmDeviceInfo(), tmpFile, isNewFile);
                    }

                    mLastTimes = currentTime;
                }
            }
        });

        if (types.contains(OBSourceInterface.TYPE_COLOR)) {
            //  Log.v("onFrameReady", "colorFrame");
            ColorFrame colorFrame = ColorFrame.get(frame);
            //   Log.d(TAG, "onFrameReady:colorFrame " + colorFrame.isValid());
            if (!colorFrame.isValid()) {
                return;
            }
            obSourceInterface.onImageFrame(OBSourceInterface.TYPE_COLOR, colorFrame, false);
        }

        if (types.contains(OBSourceInterface.TYPE_DEPTH)) {
            //   Log.v("onFrameReady", "DepthFrame");
            DepthFrame depthFrame = DepthFrame.get(frame);
            if (!depthFrame.isValid()) {
                return;
            }
            //  Log.d(TAG, "onFrameReady:depthFrame " + depthFrame.isValid());
            obSourceInterface.onImageFrame(OBSourceInterface.TYPE_DEPTH, depthFrame, false);
        }

        if (types.contains(OBSourceInterface.TYPE_BODY)) {
            //   Log.v("onFrameReady", "BodyFrame");
            try {
                BodyFrame bodyFrame = BodyFrame.get(frame);
                //   Log.d(TAG, "onFrameReady:bodyFrame " + bodyFrame.isValid());
                if (!bodyFrame.isValid()) {
                    return;
                }
                mHasUser = hasAnyBody(frame);
                obSourceInterface.onBodyFrame(OBSourceInterface.TYPE_BODY, bodyFrame, mHasUser);
            } catch (Exception ex) {
                //  Log.e("tzh", "onFrameReady: ", ex.fillInStackTrace());
            }
        }

        if (types.contains(OBSourceInterface.TYPE_MASKED_COLOR)) {
            MaskedColorFrame maskedColorFrame = MaskedColorFrame.get(frame);
            if (!maskedColorFrame.isValid()) {
                return;
            }
            mHasUser = hasAnyBody(frame);
            obSourceInterface.onImageFrame(OBSourceInterface.TYPE_MASKED_COLOR, maskedColorFrame, mHasUser);
        }

        if (types.contains(OBSourceInterface.TYPE_COLOR_ZEDBODY)) {
            ColorizedBodyFrame colorizedBodyFrame = ColorizedBodyFrame.get(frame);
            if (!colorizedBodyFrame.isValid()) {
                return;
            }
            mHasUser = hasAnyBody(frame);
            obSourceInterface.onImageFrame(OBSourceInterface.TYPE_COLOR_ZEDBODY, colorizedBodyFrame, mHasUser);
        }
    }

    private boolean hasAnyBody(ReaderFrame frame) {
        BodyFrame bodyFrame = BodyFrame.get(frame);
        return bodyFrame.getBodies() != null && bodyFrame.getBodies().iterator().hasNext();
    }
}
