package com.orbbec.obviewer2.model;

import android.content.Context;
import android.util.Log;
import android.view.View;

import com.lzh.easythread.EasyThread;
import com.orbbec.astra.Astra;
import com.orbbec.astra.BodyFrame;
import com.orbbec.astra.BodyStream;
import com.orbbec.astra.BodyTracking;
import com.orbbec.astra.BodyTrackingFeatureFlags;
import com.orbbec.astra.ColorStream;
import com.orbbec.astra.ColorizedBodyStream;
import com.orbbec.astra.DepthFrame;
import com.orbbec.astra.DepthStream;
import com.orbbec.astra.ImageFrame;
import com.orbbec.astra.ImageStreamMode;
import com.orbbec.astra.MaskedColorStream;
import com.orbbec.astra.SkeletonOptimization;
import com.orbbec.astra.SkeletonProfile;
import com.orbbec.astra.StreamReader;
import com.orbbec.astra.StreamSet;
import com.orbbec.astra.android.AstraAndroidContext;
import com.orbbec.obviewer2.callback.ViewCallBack;
import com.orbbec.obviewer2.listener.OBAstraDeviceManagerListener;
import com.orbbec.obviewer2.listener.OBFrameListener;
import com.orbbec.obviewer2.present.OpenInterface;
import com.orbbec.obviewer2.util.ThreadManager;

import java.util.ArrayList;

import static com.orbbec.astra.PixelFormat.RGB888;

/**
 * @author tanzhuohui
 * @date 2018/12/25
 */
public class OBSourceImp implements OBSourceInterface {
    private static final String TAG = "OBSourceImp";
    private OpenInterface openInterface;
    private StreamSet mStreamSet;
    private StreamReader mStreamReader;
    private ColorStream mColorStream;
    private DepthStream mDepthStream;
    private BodyStream mBodyStream;
    private MaskedColorStream mMaskedColorStream;
    private ColorizedBodyStream mColorizedBodyStream;
    boolean mExit = false;
    private EasyThread mEasyThread;
    private Thread mThread;
    private Runnable mRunnable;
    private OBFrameListener mFrameListener;
    private Context context;
    private AstraAndroidContext astraAndroidContext;


    private boolean mDepthConfigured = false;
    private int mDepthWidth = 640;
    private int mDepthHeight = 480;

    private int mColorWidth = 640;
    private int mColorHeight = 480;

    public OBSourceImp(OpenInterface openInterface) {
        this.openInterface = openInterface;
    }

    //todo
    //mBodyStream.setSkeletonProfile();  setBodyFeatures
    @Override
    public void initDevice(Context context) {
        this.context = context;
        astraAndroidContext = new AstraAndroidContext(context, new OBAstraDeviceManagerListener(openInterface));
        astraAndroidContext.initialize();
      //  BodyTracking.setLicense("QwvXpEEymPfFzud1kR6r0ODxu4TfjSUv6NnEDGLmU/IT2jhLPdFjFh8GRJXHYjvETAJwr7tp4axccZUeJSzaCE5hbWU9WmhvdUhhbm1pbnxPcmc9U2hlblpoZW5DaGFuZ3R1U2h1bWFLZWppfENvbW1lbnQ9e2NvbW1lbnQ6LGN1c3RvbWVyX3R5cGU6aW1wb3J0YW50LGxpY2Vuc2VfdHlwZTpvZmZsaW5lLHZlcnNpb246MX18RXhwaXJhdGlvbj0xOTAyOTEyNzA2");
        astraAndroidContext.openAllDevices();
        mEasyThread = ThreadManager.getIO();
    }

    @Override
    public void onDeviceOpened() {
        mFrameListener = new OBFrameListener(openInterface, this, context);
        mStreamSet = StreamSet.open();
        Log.d(TAG, "onDeviceOpened: mStreamSet");
        mStreamReader = mStreamSet.createReader();

        try {
            mDepthStream = DepthStream.get(mStreamReader);
        } catch (Exception e) {
        }

        try {
            mColorStream = ColorStream.get(mStreamReader);
        } catch (Exception e) {
        }

        try {
            mBodyStream = BodyStream.get(mStreamReader);
        } catch (Exception e) {
        }

        try {
            mMaskedColorStream = MaskedColorStream.get(mStreamReader);
        } catch (Exception e) {
        }

        try {
            mColorizedBodyStream = ColorizedBodyStream.get(mStreamReader);
        } catch (Exception e) {
        }

        mStreamReader.addFrameListener(mFrameListener);

        openInterface.isExistsColorCamera(mDepthStream.getUsbDeviceInfo().getProductId());
       // Log.d(TAG, "onDeviceOpenedlgp: "+ mDepthStream.getUsbDeviceInfo().getProductId());

    }

    public void setTypes(ArrayList<Integer> types) {
        mFrameListener.setTypes(types);
    }

    @Override
    public Object getStream(int type) {
        switch (type) {
            case TYPE_COLOR:
                return mColorStream;
            case TYPE_DEPTH:
                return mDepthStream;
            case TYPE_BODY:
                return mBodyStream;
            case TYPE_COLOR_ZEDBODY:
                return mColorizedBodyStream;
            case TYPE_MASKED_COLOR:
                return mMaskedColorStream;
            default:
                return null;
        }
    }

    @Override
    public void startStream(int type) {
        if (mDepthConfigured == false) {
            configureDepth();
        }

        synchronized (this) {
            switch (type) {
                case TYPE_COLOR:
                    startColorStream();
                    break;
                case TYPE_DEPTH:
                    startDepthStream();
                    break;
                case TYPE_BODY:
                    startBodyStream();
                    break;
                case TYPE_COLOR_ZEDBODY:
                    startColorizedBodyStream();
                    break;
                case TYPE_MASKED_COLOR:
                    startMaskedColorStream();
                    break;
                default:
                    break;
            }
        }
    }

    @Override
    public void stopStream(int type) {
        synchronized (this) {
            switch (type) {
                case TYPE_COLOR:
                    stopColorStream();
                    break;
                case TYPE_DEPTH:
                    stopDepthStream();
                    break;
                case TYPE_BODY:
                    stopBodyStream();
                    break;
                case TYPE_COLOR_ZEDBODY:
                    stopColorizedBodyStream();
                    break;
                case TYPE_MASKED_COLOR:
                    stopMaskedColorStream();
                    break;
                default:
                    break;
            }
        }
    }

    @Override
    public void setResolution(int type, int width, int height) {

            switch (type) {
                case TYPE_DEPTH: {
                    mDepthWidth = width;
                    mDepthHeight = height;
                    stopDepthStream();
                    mDepthConfigured = false;
                    break;
                }
                case TYPE_COLOR:
                    Log.d(TAG, "setResolution: " + width + " : " + height);
                    mColorWidth = width;
                    mColorHeight = height;
                    stopColorStream();
                    break;
                default:
                    break;

        }
    }

    @Override
    public synchronized void setMirror(int type, boolean isMirror) {

        switch (type) {
            case TYPE_COLOR:
                mColorStream.setMirroring(isMirror);
                break;
            case TYPE_DEPTH:
                mDepthStream.setMirroring(isMirror);
                break;
            default:
                break;
        }
    }


    @Override
    public void onRelease() {
        if (!mExit) {
            mExit = true;
            mThread = Thread.currentThread();
            if (mRunnable != null) {
                ThreadManager.join(mRunnable);
            }
        }

        if (mMaskedColorStream != null) {
            mMaskedColorStream.stop();
            mMaskedColorStream = null;
        }
        if (mColorizedBodyStream != null) {
            mColorizedBodyStream.stop();
            mColorizedBodyStream = null;
        }

        stopColorDepthBodyStream();

        if (mStreamReader != null) {
            mStreamReader.removeFrameListener(mFrameListener);
            mStreamReader.destroy();
        }

        if (astraAndroidContext != null) {
            astraAndroidContext.terminate();
        }

    }

    @Override
    public void onImageFrame(int type, ImageFrame imageFrame, boolean hasUser) {
      //  Log.d(TAG, "onImageFrame: "+imageFrame.isValid());
        //  Log.d("onFrameReady", "onImageFrame: " + type);
        openInterface.onImageFrame(type, imageFrame, hasUser);
    }

    @Override
    public void onBodyFrame(int type, BodyFrame imageFrame, boolean hasUser) {
        openInterface.onBodyFrame(type, imageFrame, hasUser);
    }

    @Override
    public void onBodyAndDepthFrame(int type, BodyFrame bodyFrame, DepthFrame imageFrame, boolean hasUser) {
        openInterface.onBodyAndDepthFrame(type, bodyFrame, imageFrame, hasUser);
    }


    @Override
    public void setSkeletonProfile(SkeletonProfile profile) {
        if (mStreamSet.getIsAvailable()) {
            mBodyStream = BodyStream.get(mStreamReader);
            Log.d(TAG, "startBodyStream profile: " + profile);
            switch (profile) {
                case FULL:
                    mBodyStream.setSkeletonProfile(SkeletonProfile.FULL);
                    Log.d(TAG, "startBodyStream: FULL");
                    break;
                case UPPER_BODY:
                    mBodyStream.setSkeletonProfile(SkeletonProfile.UPPER_BODY);
                    Log.d(TAG, "startBodyStream: UPPER_BODY");
                    break;
                case BASIC:
                    mBodyStream.setSkeletonProfile(SkeletonProfile.BASIC);
                    Log.d(TAG, "startBodyStream: BASIC");
                    break;
            }


        }


    }

    @Override
    public void setDefaultBodyFeatures(BodyTrackingFeatureFlags features) {
        if (mStreamSet.getIsAvailable()) {
            mBodyStream = BodyStream.get(mStreamReader);
            Log.d(TAG, "startBodyStream profile: " + features);
            switch (features) {
                case SEGMENTATION:
                    mBodyStream.setDefaultBodyFeatures(BodyTrackingFeatureFlags.SEGMENTATION);
                    break;
                case JOINTS:
                    mBodyStream.setDefaultBodyFeatures(BodyTrackingFeatureFlags.JOINTS);
                    break;
                case HAND_POSES:
                    mBodyStream.setDefaultBodyFeatures(BodyTrackingFeatureFlags.HAND_POSES);
                    break;
                default:
                    break;
            }
        }
    }


    private void startColorStream() {
        Log.i("startColorStream", "startColorStream Begin");
        configureColor();

        //3.开始读取数据
        if (mColorStream != null) {
            Log.d("startColorStream", "startColorStream: start");
            mColorStream.start();
        }

        //4.数据一直在更新，只要有任何一个流打开，就更新数据。
        updateDataThread();
        Log.i("startColorStream", "startColorStream End");
    }

    private void startDepthStream() {
        Log.i("startDepthStream", "startDepthStream Begin");
        if (mDepthStream != null) {
            Log.d("startColorStream", "startDepthStream: start");
            mDepthStream.setRegistration(true);
            mDepthStream.start();
        }
        //4.开始Updata数据
        updateDataThread();
        Log.i("startDepthStream", "startDepthStream End");
    }

    private void startBodyStream() {
        Log.i("startBodyStream", "startBodyStream Begin");
        if (mBodyStream != null) {
            mBodyStream.setSkeletonOptimization(SkeletonOptimization.MINIMIZE_MEMORY);
            mBodyStream.start();
        }

        //4.数据一直在更新，只要有任何一个流打开，就更新数据。
        updateDataThread();
        Log.i("startBodyStream", "startBodyStream End");
    }

    private void startMaskedColorStream() {
        Log.i("startMaskedColorStream", "startMaskedColorStream Begin");

        startBodyStream();//use body frame to check if detected user.
        if (mMaskedColorStream != null) {
            mMaskedColorStream.start();
        }

        //4.数据一直在更新
        updateDataThread();
        Log.i("startMaskedColorStream", "startMaskedColorStream End");
    }

    private void startColorizedBodyStream() {
        Log.i("ColorizedBodyStream", "startColorizedBodyStream Begin");
        startBodyStream();//use body frame to check if detected user.
        if (mColorizedBodyStream != null) {
            mColorizedBodyStream.start();
        }

        //4.数据一直在更新
        updateDataThread();
        Log.i("ColorizedBodyStream", "startColorizedBodyStream End");
    }


    private void stopColorStream() {
        Log.i("stopColorStream", "stopColorStream Begin");
        if (mColorStream != null) {
            mColorStream.stop();
        }
        Log.i("stopColorStream", "stopColorStream Begin");
    }

    private void stopDepthStream() {
        Log.i("stopDepthStream", "stopDepthStream Begin");
        //1.停止读取数据
        if (mDepthStream != null) {
            mDepthStream.stop();
        }
        Log.i("stopDepthStream", "stopDepthStream End");
    }

    private void stopBodyStream() {
        Log.i("stopBodyStream", "stopBodyStream Begin");
        try {
            //1.停止读取数据
            if (mBodyStream != null) {
                //TODO:13版本多次打开关闭仍然会出现画面卡住。
                mBodyStream.stop();
            }
        } catch (Exception ex) {
            Log.e("tzh", "stopBodyStream: ", ex.fillInStackTrace());
        }
        Log.i("stopBodyStream", "stopBodyStream End");
    }

    private void stopMaskedColorStream() {
        Log.i("stopMaskedColorStream", "stopMaskedColorStream Begin");
        if (mMaskedColorStream != null) {
            mMaskedColorStream.stop();
        }
        Log.i("stopMaskedColorStream", "stopMaskedColorStream End");
    }

    private void stopColorizedBodyStream() {
        Log.i("stopColorizedBodyStream", "stopColorizedBodyStream Begin");
        if (mColorizedBodyStream != null) {
            mColorizedBodyStream.stop();
        }
        Log.i("stopColorizedBodyStream", "stopColorizedBodyStream End");
    }


    private void configureColor() {
        if (mColorStream == null) {
            Log.e(TAG, "configureColor failed, mColorStream is null");
            return;
        }
        Log.d(TAG, "configureColor: "+mColorStream.getIsAvailable());

        ImageStreamMode newMode = null;
        for (ImageStreamMode mode : mColorStream.getAvailableModes()) {
            if (mode.getWidth() == mColorWidth && mode.getFormat() == RGB888 && mode.getFramesPerSecond() == 30) {
                newMode = mode;
            }
        }
        if (newMode != null) {
            Log.d("configureColor", "configureColor: " + newMode.toString());
            mColorStream.setMode(newMode);
        }
    }

    private void configureDepth() {
        if (mDepthStream == null) {
            Log.e(TAG, "configureDepth failed, mDepthStream is null");
            return;
        }

        ImageStreamMode newMode = null;
        for (ImageStreamMode mode : mDepthStream.getAvailableModes()) {
            if (mode.getWidth() == mDepthWidth && mode.getHeight() == mDepthHeight) {
                newMode = mode;
            }
        }
        if (newMode != null) {
            mDepthStream.setMode(newMode);
        }
        mDepthConfigured = true;
    }


    private void stopColorDepthBodyStream() {
        Log.i("stopColorDepthBody", "Begin");
        if (mColorStream != null) {
            mColorStream.stop();
        }
        if (mDepthStream != null) {
            mDepthStream.stop();
        }
        if (mBodyStream != null) {
            mBodyStream.stop();
        }
        Log.i("stopColorDepthBody", "End");
    }


    private void updateDataThread() {
        if (mRunnable == null) {
            mExit = false;

            mRunnable = new Runnable() {
                @Override
                public void run() {
                    while (!mExit) {
                        synchronized (OBSourceImp.this) {
                            Astra.update();
                        }
                        // Log.v("updateDataThread", "running");
                        try {
                            Thread.sleep(10);
                        } catch (InterruptedException e) {
                            e.printStackTrace();
                        }
                    }
                    // Log.i("updateDataThread", "exit");

                    ThreadManager.wakeup(mThread);
                }
            };
            mEasyThread.execute(mRunnable);
        }
    }

}
