package com.orbbec.obviewer2.present;

import android.content.Context;
import android.util.Log;

import com.orbbec.astra.Body;
import com.orbbec.astra.BodyFrame;
import com.orbbec.astra.BodyTrackingFeatureFlags;
import com.orbbec.astra.ColorFrame;
import com.orbbec.astra.ColorizedBodyFrame;
import com.orbbec.astra.DepthFrame;
import com.orbbec.astra.ImageFrame;
import com.orbbec.astra.Joint;
import com.orbbec.astra.JointType;
import com.orbbec.astra.MaskedColorFrame;
import com.orbbec.astra.SkeletonProfile;
import com.orbbec.astra.Vector2D;
import com.orbbec.obviewer2.callback.ViewCallBack;
import com.orbbec.obviewer2.model.OBSourceImp;
import com.orbbec.obviewer2.model.OBSourceInterface;
import com.orbbec.obviewer2.util.FPSMeter;
import com.orbbec.obviewer2.util.Utils;
import com.orbbec.obviewer2.view.GlBodyRenderer;

import java.nio.ByteBuffer;
import java.nio.ShortBuffer;
import java.util.ArrayList;

/**
 * @author tanzhuohui
 * @date 2018/12/25
 */
public class ObViewWallPresentImp implements OpenInterface {
    private ArrayList<Integer> types;
    private Context context;
    private static final String TAG = "ObViewWallPresentImp";
    private OBSourceImp obSourceInterface;
    private ViewCallBack viewCallBack;
    private GlBodyRenderer mGLBodyRenderer;
    private float mWRatio = 1.0f;
    private float mHRatio = 1.0f;

    public ObViewWallPresentImp(Context context) {
        this.obSourceInterface = new OBSourceImp(this);
        this.context = context;
        types = new ArrayList<>();
        Utils.getInstance().setTypes(types);
    }

    public void setmGLBodyRenderer(GlBodyRenderer mGLBodyRenderer) {
        this.mGLBodyRenderer = mGLBodyRenderer;
    }

    public void setViewCallBack(ViewCallBack viewCallBack) {
        this.viewCallBack = viewCallBack;
    }

    @Override
    public void addType(int type) {
        types.add(type);
        obSourceInterface.setTypes(types);
        Utils.getInstance().setTypes(types);
    }

    @Override
    public void cutType(int type) {
        types.remove((Object) type);
        obSourceInterface.setTypes(types);
        Utils.getInstance().setTypes(types);
    }

    @Override
    public void onImageFrame(int type, ImageFrame imageFrame, boolean hasUser) {
        switch (type) {
            case OBSourceInterface.TYPE_COLOR:
                //  Log.d("onFrameReady", "onImageFrame: OBPresentImpl");
                onColorFrame((ColorFrame) imageFrame);
                break;
            case OBSourceInterface.TYPE_DEPTH:
                onDepthFrame((DepthFrame) imageFrame);
                break;
            case OBSourceInterface.TYPE_COLOR_ZEDBODY:
                //Log.d(TAG, "onImageFrame: ");
                onColorizedBodyFrame((ColorizedBodyFrame) imageFrame, hasUser);
                break;
            case OBSourceInterface.TYPE_MASKED_COLOR:
                onMaskedColorFrame((MaskedColorFrame) imageFrame, hasUser);
                break;
            default:
                break;
        }
    }

    @Override
    public void onBodyFrame(int type, BodyFrame imageFrame, boolean hasUser) {
        onBodyFrame(imageFrame);
    }

    @Override
    public void onBodyAndDepthFrame(int type, BodyFrame imageFrame, DepthFrame depthFrame, boolean hasUser) {
        onBodyDepthFrame(imageFrame, depthFrame);
    }

    @Override
    public void calculateUpdateData(FPSMeter colorMeter) {
        viewCallBack.calculateUpdateData(colorMeter);
    }

    @Override
    public void calculateShowTime() {
        viewCallBack.calculateShowTime();
    }

    @Override
    public void setSkeletonProfile(SkeletonProfile profile) {
        obSourceInterface.setSkeletonProfile(profile);
    }

    @Override
    public void setDefaultBodyFeatures(BodyTrackingFeatureFlags features) {
        obSourceInterface.setDefaultBodyFeatures(features);
    }

    @Override
    public void onNoDevice() {
        viewCallBack.onNoDevice();
    }

    @Override
    public void isExistsColorCamera(int pid) {
        viewCallBack.isExistsColorCamera(pid);
    }

    @Override
    public void initDevice() {
        obSourceInterface.initDevice(context);
    }


    @Override
    public void onDeviceOpened() {
        obSourceInterface.onDeviceOpened();
        viewCallBack.isCameraOpen();
    }

    @Override
    public void onDeviceFailed() {
        Log.d(TAG, "onDeviceFailed: ");
        // TODO: 2018/12/25 设备打开失败提示,并不会调用
        viewCallBack.CameraOpenFailed();
    }

    @Override
    public Object getStream(int type) {
        return obSourceInterface.getStream(type);
    }

    @Override
    public void startStream(int type) {
        obSourceInterface.startStream(type);
    }

    @Override
    public void stopStream(int type) {
        obSourceInterface.stopStream(type);
    }

    @Override
    public void setResolution(int type, int width, int height) {
        obSourceInterface.setResolution(type, width, height);
    }

    @Override
    public void setMirror(int type, boolean isMirror) {
        obSourceInterface.setMirror(type, isMirror);
    }

    @Override
    public void onRelease() {
        obSourceInterface.onRelease();
    }

    private void onColorFrame(ColorFrame colorFrame) {
        if (colorFrame == null) {
            Log.v("onColorFrame", "onColorFrame == null");
            return;
        }

        // Log.v("onFrameReady", "ColorFrame : " + colorFrame.getWidth() + "*" + colorFrame.getHeight() + ", PixelFormat = " + colorFrame.getPixelFormat());
        ByteBuffer colorData = colorFrame.getByteBuffer();
        viewCallBack.update(OBSourceInterface.TYPE_COLOR, colorFrame.getWidth(), colorFrame.getHeight(), colorData);
    }

    private void onDepthFrame(DepthFrame depthFrame) {
        if (depthFrame == null) {
            return;
        }
        //  Log.v("depthFrame", "depthFrame" + depthFrame.getWidth() + depthFrame.getHeight() + depthFrame.getPixelFormat());
        ShortBuffer depthData = depthFrame.getDepthBuffer();
        viewCallBack.updateDepth(depthFrame.getWidth(), depthFrame.getHeight(), depthData);
        //  Log.v("depthFrame", "depthFrame processed");
    }

    private void onMaskedColorFrame(MaskedColorFrame maskedColorFrame, boolean mHasUser) {
        if (maskedColorFrame == null) {
            return;
        }

        //  Log.v("maskedColorFrame", "maskedColorFrame" + maskedColorFrame.getWidth() + maskedColorFrame.getHeight()
        //      + maskedColorFrame.getPixelFormat() + maskedColorFrame.getBytesPerPixel());
        ByteBuffer maskedColorFrameByteBuffer = maskedColorFrame.getByteBuffer();

        if (mHasUser) {
            //   Log.v("onMaskedColorFrame", "hasUser");
            viewCallBack.update(OBSourceInterface.TYPE_MASKED_COLOR, maskedColorFrame.getWidth(), maskedColorFrame.getHeight(), maskedColorFrameByteBuffer);
        }
    }

    private void onColorizedBodyFrame(ColorizedBodyFrame colorizedBodyFrame, boolean mHasUser) {
        if (colorizedBodyFrame == null) {
            return;
        }
        // Log.v("maskedColorFrame", "maskedColorFrame" + colorizedBodyFrame.getWidth() + colorizedBodyFrame.getHeight()
        //        + colorizedBodyFrame.getPixelFormat() + colorizedBodyFrame.getBytesPerPixel());
        ByteBuffer colorizedBodyFrameByteBuffer = colorizedBodyFrame.getByteBuffer();

        if (mHasUser) {
            Log.v("onColorizedBodyFrame", "hasUser");
            viewCallBack.update(OBSourceInterface.TYPE_COLOR_ZEDBODY, colorizedBodyFrame.getWidth(), colorizedBodyFrame.getHeight(), colorizedBodyFrameByteBuffer);
            //TODO:640*480分辨率的绘制：
            // 如果先打开深度流，再打开label图，则640*480显示正常；
            // 如果直接打开Label图（160*120）再打开深度流（640*480），则640*480显示异常（布局是按照160的设置）。
        }
    }

    private void onBodyFrame(BodyFrame bodyFrame) {
        if (bodyFrame == null) {
            return;
        }
        Iterable<Body> bodies = bodyFrame.getBodies();
        if (bodies == null) {
            viewCallBack.updateBodyData(-1, null, -1);
            return;
        }

        mGLBodyRenderer.clear();

        int index = 0;
        Body body = bodies.iterator().next();

        //   for (Body body : bodies) {
        Joint[] joints = body.getJoints();
        //  Log.d(TAG, "onBodyFrame: "+joints.length);
        int leftCode = body.getHandPoseInfo().getLeftHand().getCode();
        int rightCode = body.getHandPoseInfo().getRightHand().getCode();

        mGLBodyRenderer.prepareBuffer(index);
        mGLBodyRenderer.baby(joints, leftCode, rightCode);
        for (Joint joint : joints) {
            //default 160 * 120
            //160*120 ：世界坐标
            //  Log.d(TAG, "onBodyFrame: "+joint.getType().getCode()+ " : "+joint.getWorldPosition().getX());
            mGLBodyRenderer.putVertex(joint.getType().getCode(), joint.getWorldPosition().getX() / 4 * mWRatio,
                    joint.getWorldPosition().getY() / 4 * -mHRatio, 0);
            if (joint.getType() == JointType.MID_SPINE) {
                joint.getWorldPosition().getZ();
                viewCallBack.updateBodyData(index, joint.getDepthPosition(), (int) joint.getWorldPosition().getZ());
            }
        }
        mGLBodyRenderer.bindBuffer();
        index++;
        // viewCallBack.update(OBSourceInterface.TYPE_BODY  , bodyFrame.getWidth(), bodyFrame.getHeight(), null);
        viewCallBack.update(OBSourceInterface.TYPE_BODY, bodyFrame.getBodyMask().getWidth(), bodyFrame.getBodyMask().getHeight(), null);
        //  }
    }

    private void onBodyDepthFrame(BodyFrame bodyFrame, DepthFrame depthFrame) {
        if (bodyFrame == null) {
            return;
        }

        Iterable<Body> bodies = bodyFrame.getBodies();
        if (bodies == null) {
            viewCallBack.updateBodyData(-1, null, -1);
            return;
        }

        mGLBodyRenderer.clear();
        ShortBuffer buffer = depthFrame.getDepthBuffer();
        int index = 0;
        for (Body body : bodies) {
            Joint[] joints = body.getJoints();
            mGLBodyRenderer.prepareBuffer(index);
            for (Joint joint : joints) {
                //default 160 * 120
                //160*120 ：世界坐标
                mGLBodyRenderer.putVertex(joint.getType().getCode(), joint.getWorldPosition().getX() / 4 * mWRatio,
                        joint.getWorldPosition().getY() / 4 * -mHRatio, 0);
                if (joint.getType() == JointType.MID_SPINE) {
                    joint.getWorldPosition().getZ();
                    Vector2D pos = joint.getDepthPosition();
                    int x = (int) pos.getX();
                    int y = (int) pos.getY();
                    viewCallBack.updateBodyData(index, pos, (int) buffer.get(y * depthFrame.getWidth() + x));
                }
            }
            mGLBodyRenderer.bindBuffer();
            index++;
            viewCallBack.update(OBSourceInterface.TYPE_BODY, 0, 0, null);
        }
    }


}
