package com.orbbec.obviewer2.util;

import android.os.Handler;
import android.os.Looper;
import android.util.Log;
import android.view.Choreographer;


import com.orbbec.obviewer2.view.FpsRecord;

import java.util.concurrent.TimeUnit;

public class FPSSamplerAction implements Choreographer.FrameCallback {

    private Choreographer mChoreographer;
    private long mLastFrameStartTime;//上一次Frame渲染的时间
    private int mFrameRendered;//Frame渲染的次数
    private int mFPS;//最新的帧率
    private Handler mHandler = new Handler(Looper.getMainLooper());

    private static final int FPS_LIMIT = 30;//最低帧率，低于这个帧代表页面不流畅
    private static final int INTERVAL = 1000;//Frame采样时间
    private static final String FPS = "FPS";
    private FpsRecord record;
    private ISamplerAction iSamplerAction;

    public FPSSamplerAction() {
        mChoreographer = Choreographer.getInstance();
        mChoreographer.postFrameCallback(this);
    }

    /**
     * 设置取消按钮的显示内容和监听
     */
    public void ISamplerAction(ISamplerAction iSamplerAction) {
        this.iSamplerAction = iSamplerAction;
    }

    @Override
    public void doFrame(long frameTimeNanos) {
        long currentTimeMillis = TimeUnit.NANOSECONDS.toMillis(frameTimeNanos);

        if (mLastFrameStartTime > 0) {
            final long timeSpace = currentTimeMillis - mLastFrameStartTime;
            mFrameRendered++;

            if (timeSpace >= INTERVAL) {
                mFPS = (int) (mFrameRendered * 1000 / timeSpace);
                mLastFrameStartTime = currentTimeMillis;
                mFrameRendered = 0;
            }

        } else {
            mLastFrameStartTime = currentTimeMillis;
        }
        mChoreographer.postFrameCallback(this);
    }

    public int dobackFps() {
        return mFPS;

    }

    public interface ISamplerAction {
        /**
         *
         */
        void doSamplerAction();
    }
}
