package com.orbbec.obviewer2.util;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.FloatBuffer;
import java.nio.ShortBuffer;
import java.util.ArrayList;

public class IndexSkeleton {
    /**
     * 骨骼点总数
     */
    public static final int BONE_POINTS = 19;

    /**
     * 表示骨骼点的维数
     * todo 表示定点属性个数 颜色RGBA 4 position(xyz) 3
     */
    public static final int COORDS_PER_VERTEX = 3;

    private static IndexSkeleton sIndexSkeleton;

    private ShortBuffer mIndexTrunk;
    private ShortBuffer mIndexTrunkUp;
    private ShortBuffer mIndexShoulder;
    private ShortBuffer mIndexLeftArm;
    private ShortBuffer mIndexRightArm;
    private ShortBuffer mIndexThighbone;
    private ShortBuffer mIndexLeftLeg;
    private ShortBuffer mIndexRightLeg;


    /**
     * 各条线的索引
     */

    // private static final short[] ARRAY_TRUNK = {0, 20, 40, 60, 80,100,120,140,160,180,200,220,240,260,276};
    //  private static final short[] ARRAY_SHOULDER = {276, 300,330,360,400,430,460,500,530,556};
    private static final short[] ARRAY_TRUNK = {0, 18, 1, 8, 9};
    private static final short[] ARRAY_TRUNK_Up = {0, 18, 1, 8};
    private static final short[] ARRAY_SHOULDER = {2, 1, 5};


    private static final short[] ARRAY_LEFT_ARM = {2, 3, 16, 4};
    private static final short[] ARRAY_RIGHT_ARM = {5, 6, 17, 7};
    private static final short[] ARRAY_THIGHBONE = {10, 9, 13};
    private static final short[] ARRAY_LEFT_LEG = {10, 11, 12};
    private static final short[] ARRAY_RIGHT_LEG = {13, 14, 15};


    private IndexSkeleton() {

        ByteBuffer byteBuffer = ByteBuffer.allocateDirect(ARRAY_TRUNK.length * 2);
        byteBuffer.order(ByteOrder.nativeOrder());
        mIndexTrunk = byteBuffer.asShortBuffer();
        mIndexTrunk.put(ARRAY_TRUNK);
        mIndexTrunk.position(0);

        byteBuffer = ByteBuffer.allocateDirect(ARRAY_TRUNK_Up.length * 2);
        byteBuffer.order(ByteOrder.nativeOrder());
        mIndexTrunkUp = byteBuffer.asShortBuffer();
        mIndexTrunkUp.put(ARRAY_TRUNK_Up);
        mIndexTrunkUp.position(0);

        byteBuffer = ByteBuffer.allocateDirect(ARRAY_SHOULDER.length * 2);
        byteBuffer.order(ByteOrder.nativeOrder());
        mIndexShoulder = byteBuffer.asShortBuffer();
        mIndexShoulder.put(ARRAY_SHOULDER);
        mIndexShoulder.position(0);

        byteBuffer = ByteBuffer.allocateDirect(ARRAY_LEFT_ARM.length * 2);
        byteBuffer.order(ByteOrder.nativeOrder());
        mIndexLeftArm = byteBuffer.asShortBuffer();
        mIndexLeftArm.put(ARRAY_LEFT_ARM);
        mIndexLeftArm.position(0);

        byteBuffer = ByteBuffer.allocateDirect(ARRAY_RIGHT_ARM.length * 2);
        byteBuffer.order(ByteOrder.nativeOrder());
        mIndexRightArm = byteBuffer.asShortBuffer();
        mIndexRightArm.put(ARRAY_RIGHT_ARM);
        mIndexRightArm.position(0);

        byteBuffer = ByteBuffer.allocateDirect(ARRAY_THIGHBONE.length * 2);
        byteBuffer.order(ByteOrder.nativeOrder());
        mIndexThighbone = byteBuffer.asShortBuffer();
        mIndexThighbone.put(ARRAY_THIGHBONE);
        mIndexThighbone.position(0);

        byteBuffer = ByteBuffer.allocateDirect(ARRAY_LEFT_LEG.length * 2);
        byteBuffer.order(ByteOrder.nativeOrder());
        mIndexLeftLeg = byteBuffer.asShortBuffer();
        mIndexLeftLeg.put(ARRAY_LEFT_LEG);
        mIndexLeftLeg.position(0);

        byteBuffer = ByteBuffer.allocateDirect(ARRAY_RIGHT_LEG.length * 2);
        byteBuffer.order(ByteOrder.nativeOrder());
        mIndexRightLeg = byteBuffer.asShortBuffer();
        mIndexRightLeg.put(ARRAY_RIGHT_LEG);
        mIndexRightLeg.position(0);


    }


    public static IndexSkeleton getInstance() {
        if (sIndexSkeleton == null) {
            synchronized (IndexSkeleton.class) {
                if (sIndexSkeleton == null) {
                    sIndexSkeleton = new IndexSkeleton();
                }
            }
        }

        return sIndexSkeleton;
    }

    public ShortBuffer getIndexTrunk() {
        return mIndexTrunk;
    }
    public ShortBuffer getIndexTrunkUp() {
        return mIndexTrunkUp;
    }

    public ShortBuffer getIndexShoulder() {
        return mIndexShoulder;
    }

    public ShortBuffer getIndexLeftArm() {
        return mIndexLeftArm;
    }

    public ShortBuffer getIndexRightArm() {
        return mIndexRightArm;
    }

    public ShortBuffer getIndexThighbone() {
        return mIndexThighbone;
    }

    public ShortBuffer getIndexLeftLeg() {
        return mIndexLeftLeg;
    }

    public ShortBuffer getIndexRightLeg() {
        return mIndexRightLeg;
    }


//    public ShortBuffer getIndexCircle() {
//        return mIndexCircle;
//    }
}
