package com.orbbec.obviewer2.util;

import android.content.Context;
import android.content.res.Resources;
import android.util.SparseArray;

import com.orbbec.obviewer2.view.ShaderUtils;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.FloatBuffer;

public class SkeletonPool {
    public static final int MAX_BUFFER = 6;

    /**
     * buffers 管理区
     */
    private static SparseArray<FloatBuffer> mBuffers = new SparseArray<>();

    public static final int MAX_HAND_TRACK = 30;
    private static FloatBuffer mHandTrackBuffer;


    public static FloatBuffer getBuffer(int index) {
        FloatBuffer buffer;
        buffer = mBuffers.get(index);
        if (buffer == null) {
            ByteBuffer byteBuffer = ByteBuffer.allocateDirect(IndexSkeleton.BONE_POINTS * IndexSkeleton.COORDS_PER_VERTEX * Float.BYTES);
            byteBuffer.order(ByteOrder.nativeOrder());
            buffer = byteBuffer.asFloatBuffer();
            mBuffers.put(index, buffer);
        }

        return buffer;
    }

    /**
     * Open GL ES programs 管理区
     */
    private static SparseArray<Integer> mPrograms = new SparseArray<>();

    public static int getProgram(Context context, int index) {
        Integer program;
        program = mPrograms.get(index);
        if (program == null) {
            Resources resources = context.getResources();
            program = ShaderUtils.createProgram(resources,
                    "shader/vertex.vert", "shader/fragment.frag");
            mPrograms.put(index, program);
        }

        return program;
    }

    /**
     * 骨架颜色管理区
     */
    private static SparseArray<Color> mColors = new SparseArray<>();
    private static Color mUnDefine = Color.valueOf(0.66f, 0.66f, 0.66f);
    private static Color mPoint = Color.valueOf(1.0f, 1.0f, 1.0f);

    static {
        initColors();
    }

    private static void initColors() {
        Color color;
        // 黄
//        color = Color.valueOf(1.0f, 1.0f, 1.0f);
//        mColors.put(0, color);
        // 橙
        color = Color.valueOf(1.0f, 0.65f, 0.0f);
        mColors.put(1, color);
        // 红
        color = Color.valueOf(1.0f, 0.0f, 0.0f);
        mColors.put(2, color);
        // 绿
        color = Color.valueOf(0.0f, 1.0f, 0.0f);
        mColors.put(3, color);
        // 青
        color = Color.valueOf(0.0f, 1.0f, 1.0f);
        mColors.put(4, color);
        // 紫
        color = Color.valueOf(0.5f, 0.0f, 0.5f);
        mColors.put(5, color);
    }

    public static Color getColor(int index) {
        Color color;
        color = mColors.get(index);
        if (color == null) {
            color = mUnDefine;
        }

        return color;
    }

    public static Color getPointColor() {
        return mPoint;
    }

    public static FloatBuffer getHandTrackBuffer() {
        if (mHandTrackBuffer == null) {
            ByteBuffer byteBuffer = ByteBuffer.allocateDirect(MAX_HAND_TRACK * IndexSkeleton.COORDS_PER_VERTEX * Float.BYTES);
            byteBuffer.order(ByteOrder.nativeOrder());
            mHandTrackBuffer = byteBuffer.asFloatBuffer();
        }
        return mHandTrackBuffer;
    }
}
