package com.orbbec.obviewer2.util;

import android.content.Context;
import android.os.Build;
import android.os.Debug;
import android.util.Log;
import android.view.View;
import android.widget.TextView;

import com.orbbec.obviewer2.model.OBSourceInterface;
import com.orbbec.obviewer2.view.GlBodyRenderer;
import com.orbbec.obviewer2.view.OpenGLMaskColorView;
import com.orbbec.obviewer2.view.OpenGlColorView;
import com.orbbec.obviewer2.view.OpenGlColorizedBodyView;
import com.orbbec.obviewer2.view.OpenGlDepthView;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.RandomAccessFile;
import java.io.UnsupportedEncodingException;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;

import static com.orbbec.obviewer2.MainActivity.TAG;

/**
 * @author tanzhuohui
 * @date 2018/12/25
 */
public class Utils {

    private long mRunTime = 0;
    private double mCPUAverage = 0;
    private double mCPUMinimum = 0;
    private double mCPUHighest = 0;
    private double mRAMAverage = 0;
    private double mRAMMinimum = 0;
    private double mRAMHighest = 0;
    private String mDeviceInfo = "";
    private Long lastCpuTime;
    private Long lastIdelCpuTime;
    private Long lastAppCpuTime;
    private RandomAccessFile procStatFile;
    private RandomAccessFile appStatFile;
    private String build = "";
    private ArrayList<Integer> types;
    private DecimalFormat mDecimalFormat;
    private String mFPSAndConsumeInfo = "";
    private boolean mSaveFrontInfo = false;

    private static DecimalFormat fpsMdt = null;
    private FPSSamplerAction action;


    private static class Instance {
        public static Utils instance = new Utils();
    }

    public static Utils getInstance() {
        return Instance.instance;
    }

    public String getBuild() {
        return build;
    }

    public boolean ismSaveFrontInfo() {
        return mSaveFrontInfo;
    }

    public String getmDeviceInfo() {
        return mDeviceInfo;
    }

    public void setTypes(ArrayList<Integer> types) {
        this.types = types;
    }

    private Utils() {
        initData();
        mDecimalFormat = new DecimalFormat("0");
        fpsMdt = (DecimalFormat) NumberFormat.getInstance(); //获得格式化类对象
        fpsMdt.applyPattern("0.0");
    }

    private void initData() {
        mRunTime = System.currentTimeMillis();
        mCPUAverage = 0;
        mCPUMinimum = 0;
        mCPUHighest = 0;
        mRAMAverage = 0;
        mRAMMinimum = 0;
        mRAMHighest = 0;
        action = new FPSSamplerAction();
    }

    /**
     * 设备信息、帧率及耗时计算
     */
    public String calculateRunTime() {
        long time = System.currentTimeMillis() - mRunTime;
        int dtime, stime;
        StringBuilder builder = new StringBuilder();
        // ms
        dtime = (int) (time % 1000);
        builder.insert(0, String.valueOf(dtime));
        if (dtime < 100) {
            builder.insert(0, "0");
        }
        if (dtime < 10) {
            builder.insert(0, "0");
        }
        builder.insert(0, " : ");
        // ss
        dtime = (int) (time / 1000);
        stime = dtime % 60;
        builder.insert(0, String.valueOf(stime));
        if (stime < 10) {
            builder.insert(0, "0");
        }
        builder.insert(0, " : ");
        // mm
        dtime = dtime / 60;
        stime = dtime % 60;
        builder.insert(0, String.valueOf(stime));
        if (stime < 10) {
            builder.insert(0, "0");
        }
        builder.insert(0, " : ");
        // hh
        dtime = dtime / 60;
        builder.insert(0, String.valueOf(dtime));
        if (dtime < 10) {
            builder.insert(0, "0");
        }
        this.build = builder.toString();
        mDeviceInfo = "";
        mDeviceInfo += "stream state" + ":";
        mDeviceInfo += getStreamState() + "\t";
        mDeviceInfo += "elapsed time:" + builder.toString() + "\t";
        return mDeviceInfo;
    }

    public String currentTime() {
        Date date = new Date(System.currentTimeMillis());
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH-mm-ss");
        String tempTime = dateFormat.format(date);
        return tempTime;
    }

    private static int CPURATESUM = 0;
    private static int CPURATENUMBER = 0;

    public void calculateCPU(TextView mTxtCpuRate, TextView mTxtAppMemory) {
        double[] cpuRate = getCPURate();
//        mCPUAverage = (mCPUAverage * mCPUCount + cpuRate[1]) / (++mCPUCount);

        if (mCPUMinimum < 0.1 || mCPUMinimum > cpuRate[1]) {
            mCPUMinimum = cpuRate[1];
        }

        if (mCPUHighest < cpuRate[1]) {
            mCPUHighest = cpuRate[1];
        }

        //1s之内(10次)的平均帧率:100毫秒调用一次
        if ((++CPURATENUMBER) % 11 != 0) {
            CPURATESUM += cpuRate[1];
        } else {
            mCPUAverage = CPURATESUM / 10;
            CPURATESUM = 0;
            CPURATENUMBER = 0;

            // mTxtCPUTotal.setText("" + String.valueOf(mDecimalFormat.format(cpuRate[0])) + "%");
            // mTxtAppMemory.setText("" + String.valueOf(mDecimalFormat.format(mCPUAverage)));
            mTxtCpuRate.setText(String.valueOf(mDecimalFormat.format(mCPUAverage)) + "%");

            mDeviceInfo += "CPU usage(%)";
            mDeviceInfo += "sum:" + String.valueOf(mDecimalFormat.format(cpuRate[0])) + " " +
                    "current:" + String.valueOf(mDecimalFormat.format(cpuRate[1])) + " " +
                    "avg:" + mDecimalFormat.format(mCPUAverage) + " " +
                    "min:" + mDecimalFormat.format(mCPUMinimum) + " " +
                    "max:" + mDecimalFormat.format(mCPUHighest) + "\t";
        }
    }

    private static int RAMRATESUM = 0;
    private static int RAMRATENUMBER = 0;

    public void calulateFPS(TextView text_fps) {
    //    Log.d(TAG, "calulateFPS: "+action.dobackFps());
        text_fps.setText(action.dobackFps()+"");


    }


    public void calculateRAM(TextView mTxtAppMemory) {
        int ram = getRunningMemory() / 1024;
//        mRAMAverage = (mRAMAverage * mRAMCount + ram) / (++mRAMCount);
        //1s之内(10次)的平均帧率:100毫秒调用一次
        if ((++RAMRATENUMBER) % 11 != 0) {
//            Log.i("++RAMRATENUMBER", "" + RAMRATENUMBER);
            RAMRATESUM += ram;
//            Log.i("RAMRATESUM", "" + RAMRATESUM);
        } else {
            mRAMAverage = RAMRATESUM / 10;
            RAMRATESUM = 0;
            RAMRATENUMBER = 0;

         //   Log.v("test00", "" + mRAMAverage);
           // Log.v("mTxtRAMCurrent", "" + ram);
            mTxtAppMemory.setText(String.valueOf(mDecimalFormat.format(ram)));
            //  mTxtRAMAverage.setText(String.valueOf(mDecimalFormat.format(mRAMAverage)));
            mDeviceInfo += "mem usage(MB)";
            mDeviceInfo += "current:" + String.valueOf(ram) + " " +
                    "avg:" + mDecimalFormat.format(mRAMAverage) + " " +
                    "min:" + mDecimalFormat.format(mRAMMinimum) + " " +
                    "max:" + mDecimalFormat.format(mRAMHighest) + "\t";
        }
    }

    private int getRunningMemory() {
        Debug.MemoryInfo memoryInfo1 = new Debug.MemoryInfo();
        Debug.getMemoryInfo(memoryInfo1);
        return memoryInfo1.getTotalPss();
    }

    private double[] getCPURate() {
        double[] result = new double[2];
        long cpuTime;
        long cpuIdelTime;
        long appTime;
        double cpuValue = 0.0D;
        double appValue = 0.0D;
        try {
            if (procStatFile == null || appStatFile == null) {
                procStatFile = new RandomAccessFile("/proc/stat", "r");
                appStatFile = new RandomAccessFile("/proc/" + android.os.Process.myPid() + "/stat", "r");
            } else {
                procStatFile.seek(0L);
                appStatFile.seek(0L);
            }
            String procStatString = procStatFile.readLine();
            String appStatString = appStatFile.readLine();
            String procStats[] = procStatString.split(" ");
            String appStats[] = appStatString.split(" ");
            cpuTime = Long.parseLong(procStats[2]) + Long.parseLong(procStats[3])
                    + Long.parseLong(procStats[4]) + Long.parseLong(procStats[5])
                    + Long.parseLong(procStats[6]) + Long.parseLong(procStats[7])
                    + Long.parseLong(procStats[8]);
            appTime = Long.parseLong(appStats[13]) + Long.parseLong(appStats[14]);
            cpuIdelTime = Long.parseLong(procStats[5]);
            if (lastCpuTime == null && lastAppCpuTime == null && lastIdelCpuTime == null) {
                lastCpuTime = cpuTime;
                lastAppCpuTime = appTime;
                lastIdelCpuTime = cpuIdelTime;
                result[0] = cpuValue;
                result[1] = appValue;
                return result;
            }
            double dCpuTime = (double) (cpuTime - lastCpuTime);
            appValue = ((double) (appTime - lastAppCpuTime) / dCpuTime) * 100D;
            cpuValue = ((double) (dCpuTime - (cpuIdelTime - lastIdelCpuTime)) / dCpuTime) * 100D;
            lastCpuTime = cpuTime;
            lastAppCpuTime = appTime;
            lastIdelCpuTime = cpuIdelTime;
        } catch (Exception e) {
            e.printStackTrace();
        }
        result[0] = cpuValue;
        result[1] = appValue;
        return result;
    }

    private static float mMinFps = 1000;
    private static float mMaxFps = 0;
    private static int mFpsSum = 0;
    private static int FRAMENUMBER = 0;

    public void calculateUpdateData(FPSMeter colorMeter, TextView mTxtUpdateDataCurrent,
                                    TextView mTxtUpdateDataAverage, TextView mTxtUpdateDataMinimum,
                                    TextView mTxtUpdateDataHighest, TextView mTxtBKUpdateDataCurrent,
                                    TextView mTxtBKUpdateDataAverage, TextView mTxtBKUpdateDataMinimum,
                                    TextView mTxtBKUpdateDataHighest) {
        float colorFps = colorMeter.getFps();
        if (colorFps < 0) {
            mTxtUpdateDataCurrent.setText("Camera FPS:");
            return;
        }
        String colorFpsStr = colorMeter.getFpsString();

        if (colorFps < mMinFps) {
            mMinFps = colorFps;
        }

        if (colorFps > mMaxFps) {
            mMaxFps = colorFps;
        }

        //1s之内(10次)的平均帧率:100毫秒调用一次
        if ((++FRAMENUMBER) % 11 != 0) {
            mFpsSum += colorFps;
        } else {
            int tmpFPSSUM = mFpsSum / 10;
            mFpsSum = 0;
            FRAMENUMBER = 0;

            mTxtUpdateDataCurrent.setText("Camera FPS: current:" + colorFpsStr);
            mTxtUpdateDataAverage.setText(", average:" + fpsMdt.format(tmpFPSSUM));
            mTxtUpdateDataMinimum.setText(", low:" + fpsMdt.format(mMinFps));
            mTxtUpdateDataHighest.setText(", high:" + fpsMdt.format(mMaxFps));

            mFPSAndConsumeInfo = "";
            mFPSAndConsumeInfo += "Camera FPS:";

            mFPSAndConsumeInfo += "current:" + colorFpsStr + " " +
                    "average:" + fpsMdt.format(tmpFPSSUM) + " " +
                    "low:" + fpsMdt.format(mMinFps) + " " +
                    "high:" + fpsMdt.format(mMaxFps) + "\t";
//            mFPSAndConsumeInfo += "后台数据更新(ms):";
            if (mMaxFps != 0 && mMinFps != 0 && colorFps > 0) {
                mTxtBKUpdateDataCurrent.setText("FPS(ms):current:" + (int) (1000 / colorFps));
                mTxtBKUpdateDataAverage.setText(", average:" + (int) (tmpFPSSUM == 0 ? 0 : 1000 / tmpFPSSUM));
                mTxtBKUpdateDataMinimum.setText(", high:" + (int) (1000 / mMaxFps));
                mTxtBKUpdateDataHighest.setText(", low:" + (int) (1000 / mMinFps));

                mFPSAndConsumeInfo += "current:" + String.valueOf(1000 / colorFps) +
                        ", average:" + fpsMdt.format(tmpFPSSUM == 0 ? 0 : 1000 / tmpFPSSUM) +
                        ", low:" + fpsMdt.format(1000 / mMinFps) +
                        ", high:" + fpsMdt.format(1000 / mMaxFps) + "\t";
            }
        }
    }

    private int mMinTime = 1000;
    private int mMaxTime = 0;
    private int mTimeSum = 0;
    private int SHOWFRAMENUMBER = 0;

    private void calculateShowTimeInfo(int showTimes, TextView mTxtFRCurrent) {
        if (showTimes > mMaxTime) {
            mMaxTime = showTimes;
        }

        if (showTimes > 0 && showTimes < mMinTime) {
            mMinTime = showTimes;
        }

        if ((++SHOWFRAMENUMBER) % 11 != 0) {
            mTimeSum += showTimes;
            mSaveFrontInfo = false;
        } else {
            int aveTimes = mTimeSum / 10;
            mTimeSum = 0;
            SHOWFRAMENUMBER = 0;

            // mTxtFRCurrent.setText("current：" + String.valueOf(showTimes));\
            //   mTxtFRCurrent.setText("" + String.valueOf(showTimes));
//            mTxtFRAverage.setText(", average：" + String.valueOf(aveTimes));
//            mTxtFRMinimum.setText(", low：" + String.valueOf(mMinTime));
//            mTxtFRHighest.setText(", high：" + String.valueOf(mMaxTime));
          //  Log.d(TAG, "calculateShowTimeInfo: " + aveTimes + " : " + mMinTime + " ： " + mMaxTime);

            mSaveFrontInfo = true;
            mFPSAndConsumeInfo += "update_texture_elapsed(ms):";
            mFPSAndConsumeInfo += "current:" + String.valueOf(showTimes) + " " +
                    "average:" + String.valueOf(aveTimes) + " " +
                    "low:" + String.valueOf(mMinTime) + " " +
                    "high:" + String.valueOf(mMaxTime) + " " + "\t";
        }
    }

    public void calculateShowTime(OpenGlColorView mColorView, OpenGlDepthView mDepthView,
                                  OpenGLMaskColorView mMaskedColorView, OpenGlColorizedBodyView mColorizedBodyView,
                                  GlBodyRenderer mGLBodyRenderer, TextView mTxtFRCurrent, TextView text_fps) {
        int tmpShowTime = 0;
        int startStremNumber = 0;


        //TODO:统一View,显示更前台更新的帧率
        if (types.contains(OBSourceInterface.TYPE_COLOR)) {
            tmpShowTime += mColorView.getDiffDrawTime();
            startStremNumber++;
        }

        if (types.contains(OBSourceInterface.TYPE_DEPTH)) {
            tmpShowTime += mDepthView.getDiffDrawTime();
            startStremNumber++;
        }

        if (types.contains(OBSourceInterface.TYPE_BODY)) {
            tmpShowTime += mGLBodyRenderer.getDiffDrawTime();

            startStremNumber++;


        }

        if (types.contains(OBSourceInterface.TYPE_MASKED_COLOR)) {
            tmpShowTime += mMaskedColorView.getDiffDrawTime();
            startStremNumber++;
        }

        if (types.contains(OBSourceInterface.TYPE_COLOR_ZEDBODY)) {
            tmpShowTime += mColorizedBodyView.getDiffDrawTime();
            startStremNumber++;
        }

        if (startStremNumber > 0) {
            int showTime = tmpShowTime / startStremNumber;
            calculateShowTimeInfo(showTime, text_fps);
        }
    }


    private String getStreamState() {
        String tmpString = "";
        if (types.contains(OBSourceInterface.TYPE_COLOR)) {
            tmpString += "color stream" + "、";
        }
        if (types.contains(OBSourceInterface.TYPE_DEPTH)) {
            tmpString += "depth stream" + "、";
        }
        if (types.contains(OBSourceInterface.TYPE_BODY)) {
            tmpString += "body stream" + "、";
        }
        if (types.contains(OBSourceInterface.TYPE_MASKED_COLOR)) {
            tmpString += "masked color" + "、";
        }
        if (types.contains(OBSourceInterface.TYPE_COLOR_ZEDBODY)) {
            tmpString += "colorized body" + "、";
        }
        if (types.contains(OBSourceInterface.TYPE_SHOW_STREAM)) {
            tmpString += "show";
        }
        return tmpString;
    }

    synchronized public void saveStateData(String filename, String str, File tmpFile,
                                           boolean isNewFile) {
        str += mFPSAndConsumeInfo;
        BufferedWriter bufferedWriter = null;
        if (isNewFile) {
            //TODO:文件已经存在情况,不会重建
            tmpFile = new File(filename);
            isNewFile = false;
        } else {
            try {
                bufferedWriter = new BufferedWriter(new OutputStreamWriter(
                        new FileOutputStream(tmpFile, true), "GBK"));
                bufferedWriter.write(str);
                bufferedWriter.write("\r\n");
            } catch (UnsupportedEncodingException e) {
                e.printStackTrace();
            } catch (IOException e) {
                e.printStackTrace();
            } finally {
                try {
                    if (bufferedWriter != null) {
                        bufferedWriter.close();
                    }
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
    }

    public void CameraOpenFailedDialog(Context context) {
        SelfDialog dialog = new SelfDialog(context);
        dialog.setCanceledOnTouchOutside(false);
        dialog.setNoOnclickListener("Confirm", new SelfDialog.onNoOnclickListener() {
            @Override
            public void onNoClick() {
                if (dialog.isShowing()) {
                    dialog.dismiss();
                }
            }
        });
        dialog.show();
    }


}
