package com.orbbec.obviewer2.view;

import android.content.Context;
import android.opengl.GLES20;
import android.opengl.GLES30;
import android.opengl.GLSurfaceView;
import android.opengl.Matrix;
import android.util.Log;
import android.util.SparseArray;

import com.orbbec.astra.Joint;
import com.orbbec.obviewer2.MainActivity;
import com.orbbec.obviewer2.util.Circle;
import com.orbbec.obviewer2.util.IndexSkeleton;
import com.orbbec.obviewer2.util.SkeletonPool;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.FloatBuffer;
import java.util.ArrayList;
import java.util.List;

import javax.microedition.khronos.egl.EGLConfig;
import javax.microedition.khronos.opengles.GL10;

import static android.content.ContentValues.TAG;
import static android.opengl.GLES10.glColor4f;
import static android.opengl.GLES20.glClearColor;
import static android.opengl.GLES20.glIsTexture;
import static com.orbbec.obviewer2.util.IndexSkeleton.BONE_POINTS;
import static com.orbbec.obviewer2.util.IndexSkeleton.COORDS_PER_VERTEX;
import static java.lang.Math.PI;
import static java.lang.Math.cos;
import static java.lang.Math.sin;

public class GlBodyRenderer implements GLSurfaceView.Renderer {
    private Context mContext;

    private float[] mViewMatrix = new float[16];
    private float[] mProjectMatrix = new float[16];
    private float[] mMVPMatrix = new float[16];

    private IndexSkeleton mIndexSkeleton;
    private int mDiffDrawTime = 0;

    private float[] mArrayVertex;
    private SparseArray<FloatBuffer> mBuffers;
    private FloatBuffer mCurrentBuffer;

    private float radius = 10.0f;
    private int n = 360;  //切割份数
    private float[] shapePos;
    private float height = 0.0f;
    private FloatBuffer vertexBufferCircle;
    private FloatBuffer vertexBufferCircle1;
    private FloatBuffer vertexBufferCircle2;
    private FloatBuffer vertexBufferCircle3;
    private FloatBuffer vertexBufferCircle4;
    private FloatBuffer vertexBufferCircle5;
    private FloatBuffer vertexBufferCircle6;
    private FloatBuffer vertexBufferCircle7;
    private FloatBuffer vertexBufferCircle8;
    private FloatBuffer vertexBufferCircle9;
    private FloatBuffer vertexBufferCircle10;
    private FloatBuffer vertexBufferCircle11;
    private FloatBuffer vertexBufferCircle12;
    private FloatBuffer vertexBufferCircle13;
    private FloatBuffer vertexBufferCircle14;
    private FloatBuffer vertexBufferCircle15;
    private FloatBuffer vertexBufferCircle16;
    private FloatBuffer vertexBufferCircle17;
    private FloatBuffer vertexBufferCircle18;
    private int leftCode;
    private int rightCode;
    float handColor[] = {0.03f, 0.06f, 0.94f, 1.0f};
    private static int modeBody;
    private static boolean isJoint = false;
    private ArrayList<Float> data;


    public GlBodyRenderer(Context context) {
        mContext = context;
        mIndexSkeleton = IndexSkeleton.getInstance();
        mBuffers = new SparseArray<>();
        mArrayVertex = new float[BONE_POINTS * COORDS_PER_VERTEX];
    }

    @Override
    public void onSurfaceCreated(GL10 gl, EGLConfig config) {
        //设置背景透明
        GLES20.glClearColor(0, 0, 0, 0);
    }

    @Override
    public void onSurfaceChanged(GL10 gl, int width, int height) {
        GLES20.glViewport(0, 0, width, height);

        float ratio = (float) width / height;
        Matrix.setLookAtM(mViewMatrix, 0, 0, 0, 200.0f,
                0, 0, 0, 0f, -1.0f, 0);
        Matrix.frustumM(mProjectMatrix, 0, ratio, -ratio, -1, 1, 1, 1000);
        Matrix.multiplyMM(mMVPMatrix, 0, mProjectMatrix, 0, mViewMatrix, 0);
    }

    @Override
    public void onDrawFrame(GL10 gl) {

        if (mBuffers == null) {
            return;
        }


        GLES20.glClear(GLES20.GL_COLOR_BUFFER_BIT | GLES20.GL_DEPTH_BUFFER_BIT);

        if (!isJoint) {
            return;
        }
        int program;
        FloatBuffer vertexBuffer;
        int size = mBuffers.size();
        for (int i = 0; i < size; i++) {
            program = SkeletonPool.getProgram(mContext, i);
            if (program == 0) {
                continue;
            }
            int key = mBuffers.keyAt(i);
            if (key <= 0) {
                continue;
            }

            GLES20.glUseProgram(program);

            vertexBuffer = mBuffers.get(key);
            if (vertexBuffer == null) {
                continue;
            }
            long beforeDrawTime = System.currentTimeMillis();
            drawProgram(gl, program, vertexBuffer, SkeletonPool.getColor(i).getComponents());
            long afterDrawTime = System.currentTimeMillis();
            mDiffDrawTime = (int) (afterDrawTime - beforeDrawTime);
            //   Log.d(MainActivity.TAG, "mDiffDrawTime: " + mDiffDrawTime);
        }
    }

    private void drawProgram(GL10 gl, int program, FloatBuffer vertexBuffer, float[] color) {

        // Log.d(TAG, "drawProgram modeBody: " + modeBody);
        int hMatrix = GLES20.glGetUniformLocation(program, "uMatrix");
        if (hMatrix < 0) {
            return;
        }
        GLES20.glUniformMatrix4fv(hMatrix, 1, false, mMVPMatrix, 0);

        int hPosition = GLES20.glGetAttribLocation(program, "aPosition");
        if (hPosition < 0) {
            return;
        }
        GLES20.glEnableVertexAttribArray(hPosition);
        GLES20.glVertexAttribPointer(hPosition, COORDS_PER_VERTEX, GLES20.GL_FLOAT, false,
                0, vertexBuffer);

        int hColor = GLES20.glGetUniformLocation(program, "uColor");
        if (hColor < 0) {
            return;
        }
        GLES20.glEnableVertexAttribArray(hColor);
        //   GLES20.glClearColor(255,255,255,255);

        GLES20.glLineWidth(4);
        GLES20.glUniform4fv(hColor, 1, color, 0);
        GLES20.glLineWidth(4);


        if (modeBody == 0) {
            GLES20.glDrawElements(GLES20.GL_LINE_STRIP, mIndexSkeleton.getIndexTrunk().limit(),
                    GLES20.GL_UNSIGNED_SHORT, mIndexSkeleton.getIndexTrunk());
            GLES20.glDrawElements(GLES20.GL_LINE_STRIP, mIndexSkeleton.getIndexShoulder().limit(),
                    GLES20.GL_UNSIGNED_SHORT, mIndexSkeleton.getIndexShoulder());
            GLES20.glDrawElements(GLES20.GL_LINE_STRIP, mIndexSkeleton.getIndexLeftArm().limit(),
                    GLES20.GL_UNSIGNED_SHORT, mIndexSkeleton.getIndexLeftArm());
            GLES20.glDrawElements(GLES20.GL_LINE_STRIP, mIndexSkeleton.getIndexRightArm().limit(),
                    GLES20.GL_UNSIGNED_SHORT, mIndexSkeleton.getIndexRightArm());

            GLES20.glDrawElements(GLES20.GL_LINE_STRIP, mIndexSkeleton.getIndexLeftLeg().limit(),
                    GLES20.GL_UNSIGNED_SHORT, mIndexSkeleton.getIndexLeftLeg());
            GLES20.glDrawElements(GLES20.GL_LINE_STRIP, mIndexSkeleton.getIndexRightLeg().limit(),
                    GLES20.GL_UNSIGNED_SHORT, mIndexSkeleton.getIndexRightLeg());
            GLES20.glDrawElements(GLES20.GL_LINE_STRIP, mIndexSkeleton.getIndexThighbone().limit(),
                    GLES20.GL_UNSIGNED_SHORT, mIndexSkeleton.getIndexThighbone());


            if (shapePos != null) {
                //绘制圆,逐个分配地址 逐个绘制
                GLES20.glVertexAttribPointer(hPosition, COORDS_PER_VERTEX, GLES20.GL_FLOAT, false, 0, vertexBufferCircle);
                GLES20.glLineWidth(4);
                GLES20.glDrawArrays(GLES20.GL_TRIANGLE_FAN, 0, shapePos.length / 3);
                GLES20.glVertexAttribPointer(hPosition, COORDS_PER_VERTEX, GLES20.GL_FLOAT, false, 0, vertexBufferCircle1);
                GLES20.glDrawArrays(GLES20.GL_TRIANGLE_FAN, 0, shapePos.length / 3);
                GLES20.glVertexAttribPointer(hPosition, COORDS_PER_VERTEX, GLES20.GL_FLOAT, false, 0, vertexBufferCircle2);
                GLES20.glDrawArrays(GLES20.GL_TRIANGLE_FAN, 0, shapePos.length / 3);

                GLES20.glVertexAttribPointer(hPosition, COORDS_PER_VERTEX, GLES20.GL_FLOAT, false, 0, vertexBufferCircle3);
                GLES20.glDrawArrays(GLES20.GL_TRIANGLE_FAN, 0, shapePos.length / 3);
                if (leftCode == 1) {
                    GLES20.glUniform4fv(hColor, 1, handColor, 0);
                } else {
                    GLES20.glUniform4fv(hColor, 1, color, 0);
                }
                GLES20.glVertexAttribPointer(hPosition, COORDS_PER_VERTEX, GLES20.GL_FLOAT, false, 0, vertexBufferCircle4);
                GLES20.glDrawArrays(GLES20.GL_TRIANGLE_FAN, 0, shapePos.length / 3);
                GLES20.glUniform4fv(hColor, 1, color, 0);
                GLES20.glVertexAttribPointer(hPosition, COORDS_PER_VERTEX, GLES20.GL_FLOAT, false, 0, vertexBufferCircle5);
                GLES20.glDrawArrays(GLES20.GL_TRIANGLE_FAN, 0, shapePos.length / 3);
                GLES20.glVertexAttribPointer(hPosition, COORDS_PER_VERTEX, GLES20.GL_FLOAT, false, 0, vertexBufferCircle6);
                GLES20.glDrawArrays(GLES20.GL_TRIANGLE_FAN, 0, shapePos.length / 3);
                if (rightCode == 1) {
                    GLES20.glUniform4fv(hColor, 1, handColor, 0);
                } else {
                    GLES20.glUniform4fv(hColor, 1, color, 0);
                }
                GLES20.glVertexAttribPointer(hPosition, COORDS_PER_VERTEX, GLES20.GL_FLOAT, false, 0, vertexBufferCircle7);
                GLES20.glDrawArrays(GLES20.GL_TRIANGLE_FAN, 0, shapePos.length / 3);
                GLES20.glUniform4fv(hColor, 1, color, 0);
                GLES20.glVertexAttribPointer(hPosition, COORDS_PER_VERTEX, GLES20.GL_FLOAT, false, 0, vertexBufferCircle8);
                GLES20.glDrawArrays(GLES20.GL_TRIANGLE_FAN, 0, shapePos.length / 3);
//下半身数据
                GLES20.glVertexAttribPointer(hPosition, COORDS_PER_VERTEX, GLES20.GL_FLOAT, false, 0, vertexBufferCircle9);
                GLES20.glDrawArrays(GLES20.GL_TRIANGLE_FAN, 0, shapePos.length / 3);

                GLES20.glVertexAttribPointer(hPosition, COORDS_PER_VERTEX, GLES20.GL_FLOAT, false, 0, vertexBufferCircle10);
                GLES20.glDrawArrays(GLES20.GL_TRIANGLE_FAN, 0, shapePos.length / 3);

                GLES20.glVertexAttribPointer(hPosition, COORDS_PER_VERTEX, GLES20.GL_FLOAT, false, 0, vertexBufferCircle11);
                GLES20.glDrawArrays(GLES20.GL_TRIANGLE_FAN, 0, shapePos.length / 3);

                GLES20.glVertexAttribPointer(hPosition, COORDS_PER_VERTEX, GLES20.GL_FLOAT, false, 0, vertexBufferCircle12);
                GLES20.glDrawArrays(GLES20.GL_TRIANGLE_FAN, 0, shapePos.length / 3);

                GLES20.glVertexAttribPointer(hPosition, COORDS_PER_VERTEX, GLES20.GL_FLOAT, false, 0, vertexBufferCircle13);
                GLES20.glDrawArrays(GLES20.GL_TRIANGLE_FAN, 0, shapePos.length / 3);

                GLES20.glVertexAttribPointer(hPosition, COORDS_PER_VERTEX, GLES20.GL_FLOAT, false, 0, vertexBufferCircle14);
                GLES20.glDrawArrays(GLES20.GL_TRIANGLE_FAN, 0, shapePos.length / 3);

                GLES20.glVertexAttribPointer(hPosition, COORDS_PER_VERTEX, GLES20.GL_FLOAT, false, 0, vertexBufferCircle15);
                GLES20.glDrawArrays(GLES20.GL_TRIANGLE_FAN, 0, shapePos.length / 3);

                GLES20.glVertexAttribPointer(hPosition, COORDS_PER_VERTEX, GLES20.GL_FLOAT, false, 0, vertexBufferCircle16);
                GLES20.glDrawArrays(GLES20.GL_TRIANGLE_FAN, 0, shapePos.length / 3);

                GLES20.glVertexAttribPointer(hPosition, COORDS_PER_VERTEX, GLES20.GL_FLOAT, false, 0, vertexBufferCircle17);
                GLES20.glDrawArrays(GLES20.GL_TRIANGLE_FAN, 0, shapePos.length / 3);

                GLES20.glVertexAttribPointer(hPosition, COORDS_PER_VERTEX, GLES20.GL_FLOAT, false, 0, vertexBufferCircle18);
                GLES20.glDrawArrays(GLES20.GL_TRIANGLE_FAN, 0, shapePos.length / 3);


            }
        }

        if (modeBody == 1) {
            //半身模式
            GLES20.glDrawElements(GLES20.GL_LINE_STRIP, mIndexSkeleton.getIndexTrunkUp().limit(),
                    GLES20.GL_UNSIGNED_SHORT, mIndexSkeleton.getIndexTrunkUp());
            GLES20.glDrawElements(GLES20.GL_LINE_STRIP, mIndexSkeleton.getIndexShoulder().limit(),
                    GLES20.GL_UNSIGNED_SHORT, mIndexSkeleton.getIndexShoulder());
            GLES20.glDrawElements(GLES20.GL_LINE_STRIP, mIndexSkeleton.getIndexLeftArm().limit(),
                    GLES20.GL_UNSIGNED_SHORT, mIndexSkeleton.getIndexLeftArm());
            GLES20.glDrawElements(GLES20.GL_LINE_STRIP, mIndexSkeleton.getIndexRightArm().limit(),
                    GLES20.GL_UNSIGNED_SHORT, mIndexSkeleton.getIndexRightArm());
            if (shapePos != null) {
                //绘制圆,逐个分配地址 逐个绘制
                GLES20.glVertexAttribPointer(hPosition, COORDS_PER_VERTEX, GLES20.GL_FLOAT, false, 0, vertexBufferCircle);
                GLES20.glLineWidth(4);
                GLES20.glDrawArrays(GLES20.GL_TRIANGLE_FAN, 0, shapePos.length / 3);
                GLES20.glVertexAttribPointer(hPosition, COORDS_PER_VERTEX, GLES20.GL_FLOAT, false, 0, vertexBufferCircle1);
                GLES20.glDrawArrays(GLES20.GL_TRIANGLE_FAN, 0, shapePos.length / 3);
                GLES20.glVertexAttribPointer(hPosition, COORDS_PER_VERTEX, GLES20.GL_FLOAT, false, 0, vertexBufferCircle2);
                GLES20.glDrawArrays(GLES20.GL_TRIANGLE_FAN, 0, shapePos.length / 3);

                GLES20.glVertexAttribPointer(hPosition, COORDS_PER_VERTEX, GLES20.GL_FLOAT, false, 0, vertexBufferCircle3);
                GLES20.glDrawArrays(GLES20.GL_TRIANGLE_FAN, 0, shapePos.length / 3);

                if (leftCode == 1) {
                    GLES20.glUniform4fv(hColor, 1, handColor, 0);
                } else {
                    GLES20.glUniform4fv(hColor, 1, color, 0);
                }

                GLES20.glVertexAttribPointer(hPosition, COORDS_PER_VERTEX, GLES20.GL_FLOAT, false, 0, vertexBufferCircle4);
                GLES20.glDrawArrays(GLES20.GL_TRIANGLE_FAN, 0, shapePos.length / 3);
                GLES20.glUniform4fv(hColor, 1, color, 0);
                GLES20.glVertexAttribPointer(hPosition, COORDS_PER_VERTEX, GLES20.GL_FLOAT, false, 0, vertexBufferCircle5);
                GLES20.glDrawArrays(GLES20.GL_TRIANGLE_FAN, 0, shapePos.length / 3);
                GLES20.glVertexAttribPointer(hPosition, COORDS_PER_VERTEX, GLES20.GL_FLOAT, false, 0, vertexBufferCircle6);
                GLES20.glDrawArrays(GLES20.GL_TRIANGLE_FAN, 0, shapePos.length / 3);
                if (rightCode == 1) {
                    GLES20.glUniform4fv(hColor, 1, handColor, 0);
                } else {
                    GLES20.glUniform4fv(hColor, 1, color, 0);
                }
                GLES20.glVertexAttribPointer(hPosition, COORDS_PER_VERTEX, GLES20.GL_FLOAT, false, 0, vertexBufferCircle7);
                GLES20.glDrawArrays(GLES20.GL_TRIANGLE_FAN, 0, shapePos.length / 3);
                GLES20.glUniform4fv(hColor, 1, color, 0);
                GLES20.glVertexAttribPointer(hPosition, COORDS_PER_VERTEX, GLES20.GL_FLOAT, false, 0, vertexBufferCircle8);
                GLES20.glDrawArrays(GLES20.GL_TRIANGLE_FAN, 0, shapePos.length / 3);
                GLES20.glVertexAttribPointer(hPosition, COORDS_PER_VERTEX, GLES20.GL_FLOAT, false, 0, vertexBufferCircle16);
                GLES20.glDrawArrays(GLES20.GL_TRIANGLE_FAN, 0, shapePos.length / 3);

                GLES20.glVertexAttribPointer(hPosition, COORDS_PER_VERTEX, GLES20.GL_FLOAT, false, 0, vertexBufferCircle17);
                GLES20.glDrawArrays(GLES20.GL_TRIANGLE_FAN, 0, shapePos.length / 3);
                GLES20.glUniform4fv(hColor, 1, color, 0);
                GLES20.glVertexAttribPointer(hPosition, COORDS_PER_VERTEX, GLES20.GL_FLOAT, false, 0, vertexBufferCircle18);
                GLES20.glDrawArrays(GLES20.GL_TRIANGLE_FAN, 0, shapePos.length / 3);
            }

        }

        if (modeBody == 2) {
            //四点模式
            GLES20.glVertexAttribPointer(hPosition, COORDS_PER_VERTEX, GLES20.GL_FLOAT, false, 0, vertexBufferCircle);
            GLES20.glDrawArrays(GLES20.GL_TRIANGLE_FAN, 0, shapePos.length / 3);
            GLES20.glVertexAttribPointer(hPosition, COORDS_PER_VERTEX, GLES20.GL_FLOAT, false, 0, vertexBufferCircle4);
            GLES20.glDrawArrays(GLES20.GL_TRIANGLE_FAN, 0, shapePos.length / 3);
            GLES20.glVertexAttribPointer(hPosition, COORDS_PER_VERTEX, GLES20.GL_FLOAT, false, 0, vertexBufferCircle7);
            GLES20.glDrawArrays(GLES20.GL_TRIANGLE_FAN, 0, shapePos.length / 3);
            GLES20.glVertexAttribPointer(hPosition, COORDS_PER_VERTEX, GLES20.GL_FLOAT, false, 0, vertexBufferCircle8);
            GLES20.glDrawArrays(GLES20.GL_TRIANGLE_FAN, 0, shapePos.length / 3);
        }

        GLES20.glDisableVertexAttribArray(hPosition);
        GLES20.glDisableVertexAttribArray(hColor);
    }


    public float[] createPositions(Joint[] joint, int k) {
        data = new ArrayList<>();
        float x = joint[k].getWorldPosition().getX() / 4 * 1.0f;
        float y = joint[k].getWorldPosition().getY() / 4 * -1.0f;
        // Log.d(TAG, "createPositions: " + x + " : " + y);
        data.add(x);            //设置圆心坐标
        data.add(y);
        data.add(0.0f);
        float angDegSpan = 360f / n;
        for (float i = 0; i < 360 + angDegSpan; i += angDegSpan) {
            data.add((float) ((radius * sin(i * PI / 180f)) + x));
            data.add((float) ((radius * cos(i * PI / 180f)) + y));
            data.add(0.0f);
        }
        float[] f = new float[data.size()];
        for (int i = 0; i < f.length; i++) {
            f[i] = data.get(i);
        }
        return f;
    }


    public void baby(Joint[] joint, int handLeftCode, int handRightCode) {
//遍历当joint数组值都为0.0时不进行绘制
        for (int i = 0; i < joint.length; i++) {
            // Log.d(TAG, "baby: " + joint[i].getWorldPosition().getX());
            if (joint[i].getWorldPosition().getX() != 0.0) {
                isJoint = true;
                break;
            } else {
                isJoint = false;
            }
        }
        leftCode = handLeftCode;
        rightCode = handRightCode;
        shapePos = createPositions(joint, 0);
        //申请底层空间
        ByteBuffer bb = ByteBuffer.allocateDirect(shapePos.length * 4);
        bb.order(ByteOrder.nativeOrder());
        //将坐标数据转换为FloatBuffer，用以传入给OpenGL ES程序
        vertexBufferCircle = bb.asFloatBuffer();
        vertexBufferCircle.put(shapePos);
        vertexBufferCircle.position(0);

        shapePos = createPositions(joint, 1);
        bb = ByteBuffer.allocateDirect(shapePos.length * 4);
        bb.order(ByteOrder.nativeOrder());
        //将坐标数据转换为FloatBuffer，用以传入给OpenGL ES程序
        vertexBufferCircle1 = bb.asFloatBuffer();
        vertexBufferCircle1.put(shapePos);
        vertexBufferCircle1.position(0);

        shapePos = createPositions(joint, 2);
        bb = ByteBuffer.allocateDirect(shapePos.length * 4);
        bb.order(ByteOrder.nativeOrder());
        //将坐标数据转换为FloatBuffer，用以传入给OpenGL ES程序
        vertexBufferCircle2 = bb.asFloatBuffer();
        vertexBufferCircle2.put(shapePos);
        vertexBufferCircle2.position(0);

        shapePos = createPositions(joint, 3);
        bb = ByteBuffer.allocateDirect(shapePos.length * 4);
        bb.order(ByteOrder.nativeOrder());
        //将坐标数据转换为FloatBuffer，用以传入给OpenGL ES程序
        vertexBufferCircle3 = bb.asFloatBuffer();
        vertexBufferCircle3.put(shapePos);
        vertexBufferCircle3.position(0);

        shapePos = createPositions(joint, 4);
        bb = ByteBuffer.allocateDirect(shapePos.length * 4);
        bb.order(ByteOrder.nativeOrder());
        //将坐标数据转换为FloatBuffer，用以传入给OpenGL ES程序
        vertexBufferCircle4 = bb.asFloatBuffer();
        vertexBufferCircle4.put(shapePos);
        vertexBufferCircle4.position(0);

        shapePos = createPositions(joint, 5);
        bb = ByteBuffer.allocateDirect(shapePos.length * 4);
        bb.order(ByteOrder.nativeOrder());
        //将坐标数据转换为FloatBuffer，用以传入给OpenGL ES程序
        vertexBufferCircle5 = bb.asFloatBuffer();
        vertexBufferCircle5.put(shapePos);
        vertexBufferCircle5.position(0);

        shapePos = createPositions(joint, 6);
        bb = ByteBuffer.allocateDirect(shapePos.length * 4);
        bb.order(ByteOrder.nativeOrder());
        //将坐标数据转换为FloatBuffer，用以传入给OpenGL ES程序
        vertexBufferCircle6 = bb.asFloatBuffer();
        vertexBufferCircle6.put(shapePos);
        vertexBufferCircle6.position(0);

        shapePos = createPositions(joint, 7);
        bb = ByteBuffer.allocateDirect(shapePos.length * 4);
        bb.order(ByteOrder.nativeOrder());
        //将坐标数据转换为FloatBuffer，用以传入给OpenGL ES程序
        vertexBufferCircle7 = bb.asFloatBuffer();
        vertexBufferCircle7.put(shapePos);
        vertexBufferCircle7.position(0);

        shapePos = createPositions(joint, 8);
        bb = ByteBuffer.allocateDirect(shapePos.length * 4);
        bb.order(ByteOrder.nativeOrder());
        //将坐标数据转换为FloatBuffer，用以传入给OpenGL ES程序
        vertexBufferCircle8 = bb.asFloatBuffer();
        vertexBufferCircle8.put(shapePos);
        vertexBufferCircle8.position(0);

        shapePos = createPositions(joint, 9);
        bb = ByteBuffer.allocateDirect(shapePos.length * 4);
        bb.order(ByteOrder.nativeOrder());
        //将坐标数据转换为FloatBuffer，用以传入给OpenGL ES程序
        vertexBufferCircle9 = bb.asFloatBuffer();
        vertexBufferCircle9.put(shapePos);
        vertexBufferCircle9.position(0);

        shapePos = createPositions(joint, 10);
        bb = ByteBuffer.allocateDirect(shapePos.length * 4);
        bb.order(ByteOrder.nativeOrder());
        //将坐标数据转换为FloatBuffer，用以传入给OpenGL ES程序
        vertexBufferCircle10 = bb.asFloatBuffer();
        vertexBufferCircle10.put(shapePos);
        vertexBufferCircle10.position(0);

        shapePos = createPositions(joint, 11);
        bb = ByteBuffer.allocateDirect(shapePos.length * 4);
        bb.order(ByteOrder.nativeOrder());
        //将坐标数据转换为FloatBuffer，用以传入给OpenGL ES程序
        vertexBufferCircle11 = bb.asFloatBuffer();
        vertexBufferCircle11.put(shapePos);
        vertexBufferCircle11.position(0);

        shapePos = createPositions(joint, 12);
        bb = ByteBuffer.allocateDirect(shapePos.length * 4);
        bb.order(ByteOrder.nativeOrder());
        //将坐标数据转换为FloatBuffer，用以传入给OpenGL ES程序
        vertexBufferCircle12 = bb.asFloatBuffer();
        vertexBufferCircle12.put(shapePos);
        vertexBufferCircle12.position(0);

        shapePos = createPositions(joint, 13);
        bb = ByteBuffer.allocateDirect(shapePos.length * 4);
        bb.order(ByteOrder.nativeOrder());
        //将坐标数据转换为FloatBuffer，用以传入给OpenGL ES程序
        vertexBufferCircle13 = bb.asFloatBuffer();
        vertexBufferCircle13.put(shapePos);
        vertexBufferCircle13.position(0);

        shapePos = createPositions(joint, 14);
        bb = ByteBuffer.allocateDirect(shapePos.length * 4);
        bb.order(ByteOrder.nativeOrder());
        //将坐标数据转换为FloatBuffer，用以传入给OpenGL ES程序
        vertexBufferCircle14 = bb.asFloatBuffer();
        vertexBufferCircle14.put(shapePos);
        vertexBufferCircle14.position(0);

        shapePos = createPositions(joint, 15);
        bb = ByteBuffer.allocateDirect(shapePos.length * 4);
        bb.order(ByteOrder.nativeOrder());
        //将坐标数据转换为FloatBuffer，用以传入给OpenGL ES程序
        vertexBufferCircle15 = bb.asFloatBuffer();
        vertexBufferCircle15.put(shapePos);
        vertexBufferCircle15.position(0);

        shapePos = createPositions(joint, 16);
        bb = ByteBuffer.allocateDirect(shapePos.length * 4);
        bb.order(ByteOrder.nativeOrder());
        //将坐标数据转换为FloatBuffer，用以传入给OpenGL ES程序
        vertexBufferCircle16 = bb.asFloatBuffer();
        vertexBufferCircle16.put(shapePos);
        vertexBufferCircle16.position(0);

        shapePos = createPositions(joint, 17);
        bb = ByteBuffer.allocateDirect(shapePos.length * 4);
        bb.order(ByteOrder.nativeOrder());
        //将坐标数据转换为FloatBuffer，用以传入给OpenGL ES程序
        vertexBufferCircle17 = bb.asFloatBuffer();
        vertexBufferCircle17.put(shapePos);
        vertexBufferCircle17.position(0);

        shapePos = createPositions(joint, 18);
        bb = ByteBuffer.allocateDirect(shapePos.length * 4);
        bb.order(ByteOrder.nativeOrder());
        //将坐标数据转换为FloatBuffer，用以传入给OpenGL ES程序
        vertexBufferCircle18 = bb.asFloatBuffer();
        vertexBufferCircle18.put(shapePos);
        vertexBufferCircle18.position(0);
    }


    public void prepareBuffer(int index) {
        mCurrentBuffer = SkeletonPool.getBuffer(index);
        mBuffers.put(index + 1, mCurrentBuffer);
    }

    public void putVertex(int type, float x, float y, float z) {
        int index = type * COORDS_PER_VERTEX;
        mArrayVertex[index] = x;
        mArrayVertex[index + 1] = y;
        mArrayVertex[index + 2] = z;
    }

    /**
     * 根据不同模式 选择不同的绘制点
     * FULL(0),
     * UPPER_BODY(1),
     * BASIC(2);
     *
     * @param mode
     */


    public void modeBody(int mode) {
        modeBody = mode;
    }

    public void bindBuffer() {
        mCurrentBuffer.put(mArrayVertex);
        mCurrentBuffer.position(0);
    }

    public void clear() {
        mBuffers.clear();

    }

    public int getDiffDrawTime() {
        return mDiffDrawTime;
    }

}
