package com.orbbec.obviewer2.view;

import android.content.Context;
import android.graphics.Color;
import android.opengl.GLES10;
import android.opengl.GLES11;
import android.opengl.GLES11Ext;
import android.opengl.GLSurfaceView;
import android.os.Build;
import android.util.AttributeSet;
import android.util.Log;

import com.orbbec.obviewer2.MainActivity;
import com.orbbec.obviewer2.listener.CheckBoxCheckChangeListenter;

import java.nio.IntBuffer;

import javax.microedition.khronos.egl.EGLConfig;
import javax.microedition.khronos.opengles.GL10;

public class OpenGlDepthView extends GLSurfaceView {

    protected int mSurfaceWidth;
    protected int mSurfaceHeight;

    // protected int mTextureWidth = 640;
    //   protected int mTextureHeight = 480;
    protected int mTextureWidth = 640;
    protected int mTextureHeight = 480;

    protected IntBuffer mTexture;

    protected int mTextureId = 0;

    private int mBaseColor = Color.WHITE;

    private static final String TAG = "OpenNIView";

    private int mDiffDrawTime = 0;

    private int mVersionInt;

    public OpenGlDepthView(Context context) {
        super(context);
        init();
    }

    public OpenGlDepthView(Context context, AttributeSet attrs) {
        super(context, attrs);
        init();

    }


    private void init() {
        mVersionInt = Build.VERSION.SDK_INT;
        setRenderer(new GLSurfaceView.Renderer() {

            @Override
            public void onSurfaceCreated(GL10 gl, EGLConfig c) {

                GLES10.glClearColor(1.0f, 1.0f, 0.0f, 1.0f);//设置背景色，黄色
                /* Disable these capabilities. */
//                final int[] gCapbilitiesToDisable = {
//                        GLES10.GL_FOG,
//                        GLES10.GL_LIGHTING,
//                        GLES10.GL_CULL_FACE,
//                        GLES10.GL_ALPHA_TEST,
//                        GLES10.GL_BLEND,
//                        GLES10.GL_COLOR_LOGIC_OP,
//                        GLES10.GL_DITHER,
//                        GLES10.GL_STENCIL_TEST,
//                        GLES10.GL_DEPTH_TEST,
//                        GLES10.GL_COLOR_MATERIAL,
//                };
//
//                for (int capability : gCapbilitiesToDisable) {
//                    GLES10.glDisable(capability);
//                }

                GLES10.glEnable(GLES10.GL_TEXTURE_2D);
                int[] ids = new int[1];
                GLES10.glGenTextures(1, ids, 0);
                mTextureId = ids[0];
                GLES10.glBindTexture(GLES10.GL_TEXTURE_2D, mTextureId);
                GLES10.glTexParameterf(GLES10.GL_TEXTURE_2D, GLES10.GL_TEXTURE_MIN_FILTER, GLES10.GL_LINEAR);
                GLES10.glTexParameterf(GLES10.GL_TEXTURE_2D, GLES10.GL_TEXTURE_MAG_FILTER, GLES10.GL_LINEAR);
                GLES10.glShadeModel(GLES10.GL_FLAT);
            }

            @Override
            public void onSurfaceChanged(GL10 gl, int w, int h) {
                synchronized (OpenGlDepthView.this) {
                    mSurfaceWidth = w;
                    mSurfaceHeight = h;
                }
            }

            @Override
            public void onDrawFrame(GL10 gl) {
                synchronized (OpenGlDepthView.this) {
                    long beforeDrawTime = System.currentTimeMillis();
                    onDrawGL();
                    long afterDrawTime = System.currentTimeMillis();

                    mDiffDrawTime = (int) (afterDrawTime - beforeDrawTime);
                  //  Log.i("onDepthDrawFrame", mDiffDrawTime + "");

                }
            }
        });

        setRenderMode(RENDERMODE_WHEN_DIRTY);
    }

    public void setBaseColor(int color) {
        mBaseColor = color;
        requestRender();
    }

    synchronized public void fillTexture(int width, int height, IntBuffer buf) {
        mTextureWidth = width;
        mTextureHeight = height;
        mTexture = buf;
        requestRender();
    }

    protected void onDrawGL() {
        if (mSurfaceWidth == 0 || mSurfaceHeight == 0 || mTexture == null) {
            Log.v(TAG, "mShortTexuture is null");
            return;
        }

        GLES10.glEnable(GLES10.GL_BLEND);
        GLES10.glBlendFunc(GLES10.GL_SRC_ALPHA, GLES10.GL_ONE_MINUS_SRC_ALPHA);
        int red = Color.red(mBaseColor);
        int green = Color.green(mBaseColor);
        int blue = Color.blue(mBaseColor);
        int alpha = Color.alpha(mBaseColor);
        GLES10.glColor4f(red / 255.f, green / 255.f, blue / 255.f, alpha / 255.f);

        GLES10.glEnable(GLES10.GL_TEXTURE_2D);

        GLES10.glBindTexture(GLES10.GL_TEXTURE_2D, mTextureId);
        //  int rect[] = {0, mCurrFrameHeight, mCurrFrameWidth, -mCurrFrameHeight};
      //  Log.d(TAG, "onDrawGL: " + mTextureHeight + mTextureWidth);
        int[] rect = {0, mTextureHeight, mTextureWidth, -mTextureHeight};

        GLES11.glTexParameteriv(GLES10.GL_TEXTURE_2D, GLES11Ext.GL_TEXTURE_CROP_RECT_OES, rect, 0);
        GLES10.glClear(GLES10.GL_COLOR_BUFFER_BIT);
        GLES10.glClearColor(1, 1, 1, 0);
        GLES10.glTexImage2D(GLES10.GL_TEXTURE_2D, 0, GLES10.GL_RGBA, mTextureWidth, mTextureHeight, 0, GLES10.GL_RGBA,
                GLES10.GL_UNSIGNED_BYTE, mTexture);
        GLES11Ext.glDrawTexiOES(0, 0, 0, mSurfaceWidth, mSurfaceHeight);
        GLES10.glDisable(GLES10.GL_TEXTURE_2D);
    }
    public int getDiffDrawTime() {
        return mDiffDrawTime;
    }

}
