package com.orbbec.obviewer2.view;

import android.content.res.Resources;
import android.opengl.GLES20;
import android.util.Log;

import java.io.IOException;
import java.io.InputStream;

public class ShaderUtils {

    private ShaderUtils() {
    }

    public static int loadShader(int shaderType, String source) {
        int shader = GLES20.glCreateShader(shaderType);
        if (shader != 0) {
            GLES20.glShaderSource(shader, source);
            GLES20.glCompileShader(shader);
            int[] compiled = new int[1];
            GLES20.glGetShaderiv(shader, GLES20.GL_COMPILE_STATUS, compiled, 0);
            if (compiled[0] != GLES20.GL_TRUE) {
                GLES20.glDeleteShader(shader);
                shader = 0;
            }
        } else {
            checkGLError("");
        }

        return shader;
    }

    public static int loadShader(Resources resources, int shaderType, String resName) {
        return loadShader(shaderType, loadFromAssetsFile(resources, resName));
    }

    public static String loadFromAssetsFile(Resources resources, String fileName) {
        StringBuilder result = new StringBuilder();
        try {
            InputStream is = resources.getAssets().open(fileName);
            int ch;
            byte[] buffer = new byte[1024];
            while ((ch = is.read(buffer)) != -1) {
                result.append(new String(buffer, 0, ch));
            }
        } catch (IOException e) {
            return null;
        }

        return result.toString().replaceAll("\\r\\n", "\n");
    }

    public static int createProgram(Resources resources, String vertexRes, String fragmentRes) {
        return createProgram(loadFromAssetsFile(resources, vertexRes), loadFromAssetsFile(resources, fragmentRes));
    }

    public static int createProgram(String vertexSource, String fragmentSource) {
        int vertex = loadShader(GLES20.GL_VERTEX_SHADER, vertexSource);
        if (vertex == 0) {
            return 0;
        }

        int fragment = loadShader(GLES20.GL_FRAGMENT_SHADER, fragmentSource);
        if (fragment == 0) {
            return 0;
        }

        int program = GLES20.glCreateProgram();
        if (program != 0) {
            GLES20.glAttachShader(program, vertex);
            GLES20.glAttachShader(program, fragment);
            GLES20.glLinkProgram(program);
            int[] linkStatus = new int[1];
            GLES20.glGetProgramiv(program, GLES20.GL_LINK_STATUS, linkStatus, 0);
            if (linkStatus[0] != GLES20.GL_TRUE) {
                GLES20.glDeleteProgram(program);
                program = 0;
            }
        }

        return program;
    }

    public static void checkGLError(String op) {
        int error;
        //错误代码不为0, 就打印错误日志, 并抛出异常
        while ((error = GLES20.glGetError()) != GLES20.GL_NO_ERROR) {
            Log.e("ES20_ERROR", op + ": glError " + error);
            throw new RuntimeException(op + ": glError " + error);
        }
    }
}